// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "cbgrid.h"
#include "colorbarbase.h"
#include "util.h"

extern "C" {
  #include "ast.h"
}

extern GridBase* astGridPtr;

CBGrid::CBGrid(Widget* p, int cc, double* ll, const char* o) 
  : cnt_(cc), lut_(ll), GridBase(p,o)
{}

int CBGrid::render()
{
  gc_ = ((ColorbarBase*)parent)->gc;
  pixmap_ = ((ColorbarBase*)parent)->pixmap;
  byteorder_ = ((ColorbarBase*)parent)->xmap->byte_order;

  return doit(X11);
}

int CBGrid::ps(int mode, int x, int y)
{
  matrix_ =  Translate(x,y);
  mode_ = mode;

  return doit(PS);
}

#ifdef _MACOSX
int CBGrid::macosx(int x, int y)
{
  matrix_ =  Translate(x,y);

  return doit(MACOSX);
}
#endif

#ifdef _WIN32
int CBGrid::win32(int x, int y)
{
  matrix_ =  Translate(x,y);

  return doit(GWIN32);
}
#endif

int CBGrid::doit(RenderMode rm)
{
  ColorbarBaseOptions* opts = 
    (ColorbarBaseOptions*)(((ColorbarBase*)parent)->options);

  astClearStatus; // just to make sure
  astBegin; // start memory management

  AstFrameSet* frameSet = NULL;
  AstCmpMap* cmp = NULL;
  AstLutMap* aa = NULL;
  AstUnitMap* bb = NULL;
  AstPlot* plot = NULL;

  if (!(frameSet = astFrameSet(astFrame(2,"Domain=WIDGET"),"")))
    goto error;

  if (!(bb = astUnitMap(1,"")))
    goto error;

  if (!opts->orientation) {
    if (!(aa = astLutMap(cnt_, lut_, 0, double(opts->width)/(cnt_-1), "")))
      goto error;
    if (!(cmp = astCmpMap(aa, bb, 0, "")))
      goto error;
  }
  else {
    if (!(aa = astLutMap(cnt_, lut_, 0, double(opts->height)/(cnt_-1), "")))
      goto error;
    if (!(cmp = astCmpMap(bb, aa, 0, "")))
      goto error;
  }

  astAddFrame(frameSet, AST__CURRENT, cmp, astFrame(2, "Domain=LUT"));

  astSet(frameSet,"Title=%s", " ");

  if (0) {
    int status = astStatus;
    astClearStatus;
    astShow(frameSet);
    astSetStatus(status);
  }

  // create astPlot
  float gbox[4];
  double pbox[4];
  int which;

  {
    int ss=opts->size;
    switch (rm) {
    case X11:
      ss++;
      break;
    case PS:
    case MACOSX:
    case GWIN32:
      break;
    }

    if (!opts->orientation) {
      which = 1;
      gbox[0] = pbox[0] = 0;
      gbox[1] = pbox[1] = 0;
      gbox[2] = pbox[2] = opts->width;
      gbox[3] = pbox[3] = ss;
    }
    else {
      which = 2;
      gbox[0] = 0;
      gbox[1] = 0;
      gbox[2] = ss;
      gbox[3] = opts->height;

      pbox[0] = 0;
      pbox[1] = opts->height;
      pbox[2] = ss;
      pbox[3] = 0;
    }

  }

  /*
  {
    float delta = (lut_[cnt_-1]-lut_[0])/100.;
    float bottom = lut_[0]+delta;
    float top = lut_[cnt_-1]-delta;

    ostringstream str;
    str << option_ 
	<< ", Bottom(" << which << ")=" << bottom 
	<< ", Top(" << which << ")=" << top << ends;
    plot = astPlot(frameSet, gbox, pbox, str.str().c_str());
  }
  */

  plot = astPlot(frameSet, gbox, pbox, option_);

  // and now create astGrid
  astGridPtr = this;
  renderMode = rm;
  astGrid(plot);

  astEnd; // now, clean up memory
  return 1;

error:
  astEnd;
  return 0;
}
