// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __colorbarbase_h__
#define __colorbarbase_h__

#include "widget.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>

// ColorbarBase

class CBGrid;
class Filter;

struct ColorbarBaseOptions {
  Tk_Item item;              // required by tk
  int x, y;                  // Coordinates of positioning point on canvas
  int width;                 // widget width
  int height;                // widget height
  Tk_Anchor anchor;          // Where to anchor widget relative to x,y
  char* cmdName;             // Suggested Tcl command name

  Widget* widget;            // pointer to widget class

  int orientation;           // 0-horizontal, 1-vertical
  int size;
  char* numopts;

  int colors;
  int minColors;
  int maxColors;
  int privateCmap;
  int privateColors;
};

class ColorbarBase : public Widget {
  friend class CBGrid;

 private:
  void updateGCs();

 protected:
  XImage* xmap;
  unsigned char* colorCells;
  int colorCount;

  CBGrid* grid;
  GC gc;

  int cnt;
  double* lut;

  int invert;

 private:
  void ps(Filter&, PSColorSpace, int, int, float);

 protected:
  virtual int initColormap() =0;
  void invalidPixmap();

#ifdef _MACOSX
  virtual void macosx(float, int, int, const Vector&, const Vector&) =0;
#endif

  int postscriptProc(int);   // generate postscript
  virtual void psHorz(Filter&, PSColorSpace, int, int) =0;
  virtual void psVert(Filter&, PSColorSpace, int, int) =0;

  virtual void reset() =0;

  virtual void updateColors() =0;
  int updatePixmap(const BBox&);

#ifdef _WIN32
  virtual void win32(float, int, int, const Vector&, const Vector&) =0;
#endif

  virtual void ximageToPixmap() =0;

 public:
  ColorbarBase(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~ColorbarBase();

  virtual void adjustCmd(float, float) =0;

  int configure(int, const char* [], int);

  virtual void getBiasCmd() =0;
  virtual void getColormapCmd() =0;
  virtual void getColormapNameCmd(int) =0;
  virtual void getContrastCmd() =0;
  virtual void getCurrentFileNameCmd() =0;
  virtual void getCurrentIDCmd() =0;
  virtual void getCurrentNameCmd() =0;
  void getInvertCmd();
  void getNumericsCmd();
  virtual void getRGBChannelCmd() =0;

  void invertCmd(int);
  virtual void ittCmd(const char*) {}
  virtual void ittCmd(int) {}

  virtual void listIDCmd() {}
  virtual void listNameCmd() {}
  virtual void listITTIDCmd() {}
  virtual void listITTNameCmd() {}
  virtual void loadCmd(const char*, const char*) {}
  virtual void loadITTCmd(const char*) {}

  virtual void mapCmd(char*) {}
  virtual void mapCmd(int) {}

  int parse(istringstream&);

  virtual void saveCmd(const char*) {}
  virtual void setColormapCmd(char*, float, float, int) {}
  virtual void setColormapCmd(int, float, float, int) {}
  virtual void setColormapCmd(float, float, float, float, float, float, int) {}
  void setColormapLevelCmd(int, float*);
  virtual void setColormapWindowCmd(char*) {}
  virtual void setRGBChannelCmd(const char*) {}

#ifdef _MACOSX
  void macosxPrintCmd();
#endif

#ifdef _WIN32
  void win32PrintCmd();
#endif
};

#endif

