// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

%pure-parser
%parse-param {ColorbarBase* cb}
%lex-param {cbFlexLexer* ll}
%parse-param {cbFlexLexer* ll}

%{
#define YYDEBUG 1

#include <stdlib.h>

#include "colorbarbase.h"

#undef yyFlexLexer
#define yyFlexLexer cbFlexLexer
#include <FlexLexer.h>

extern int cblex(void*, cbFlexLexer*);
extern void cberror(ColorbarBase*, cbFlexLexer*, const char*);

int cnt;
float levels[256];
%}

%union {
  float real;
  int integer;
  char str[1024];
}

%type <real> numeric
%type <integer> yesno
%type <integer> pscolorspace

%token <integer> INT
%token <real> REAL
%token <str> STRING

%token ADJUST_
%token BIAS_
%token BW_
%token CHANNEL_
%token CMYK_
%token COLORMAP_
%token COLORSPACE_
%token COLORS_
%token CONTRAST_
%token DEBUG_
%token GET_
%token GRAY_
%token FALSE_
%token FILE_
%token HEIGHT_
%token HIDE_
%token ID_
%token INVERT_
%token ITT_
%token LEVEL_
%token LIST_
%token LOAD_
%token MACOSX_
%token MAP_
%token N_
%token NAME_
%token NO_
%token OFF_
%token ON_
%token POSTSCRIPT_
%token PRINT_
%token RESET_
%token RESOLUTION_
%token RGB_
%token SAVE_
%token SET_
%token SHOW_
%token TRUE_
%token VERSION_
%token WIDTH_
%token WIN32_
%token WINDOW_
%token Y_
%token YES_

%%

command : DEBUG_ debug

	| ADJUST_ numeric numeric {cb->adjustCmd($2,$3);}
	| GET_ get
	| HIDE_ {cb->hideCmd();}
	| INVERT_ yesno {cb->invertCmd($2 );}
	| ITT_ itt
	| LIST_ list
	| LOAD_ load
	| MACOSX_ macosx
	| MAP_ map
	| POSTSCRIPT_ postscript
	| RESET_ {cb->resetCmd();}
	| RGB_ CHANNEL_ STRING {cb->setRGBChannelCmd($3);}
	| SAVE_ STRING {cb->saveCmd($2);}
	| SET_ set
	| SHOW_ {cb->showCmd();}
	| VERSION_ {cb->msg("Colorbar 1.0");}
	| WIN32_ win32
	;

numeric	: REAL {$$=$1;}
	| INT {$$=$1;}
	;

debug	: ON_ {yydebug=1;}
	| OFF_ {yydebug=0;}
	;

yesno	: INT {$$=($1 ? 1 : 0);}

	| YES_ {$$=1;}
	| Y_ {$$=1;}
	| ON_ {$$=1;}
	| TRUE_ {$$=1;}

	| NO_ {$$=0;}
	| N_ {$$=0;}
	| OFF_ {$$=0;}
	| FALSE_ {$$=0;}
	;

get	: BIAS_ {cb->getBiasCmd();}
	| COLORMAP_ {cb->getColormapCmd();}
	| CONTRAST_ {cb->getContrastCmd();}
	| FILE_ NAME_ {cb->getCurrentFileNameCmd();}
	| HEIGHT_ {cb->getHeightCmd();}
	| ID_ {cb->getCurrentIDCmd();}
	| INVERT_ {cb->getInvertCmd();}
	| NAME_ {cb->getCurrentNameCmd();}
	| NAME_ INT {cb->getColormapNameCmd($2);}
	| RGB_ CHANNEL_ {cb->getRGBChannelCmd();}
	| WIDTH_ {cb->getWidthCmd();}
	;

itt	: STRING {cb->ittCmd($1);}
	| INT {cb->ittCmd($1);}
	;

list	: /* empty */ {cb->listNameCmd();}
	| ID_ {cb->listIDCmd();}
	| NAME_ {cb->listNameCmd();}
	| ITT_ {cb->listITTNameCmd();}
	| ITT_ ID_ {cb->listITTIDCmd();}
	| ITT_ NAME_ {cb->listITTNameCmd();}
	;

load	: STRING {cb->loadCmd($1,NULL);}
	| STRING STRING {cb->loadCmd($1,$2);}
	| ITT_ STRING {cb->loadITTCmd($2);}
	;

macosx	: PRINT_ {
#ifdef _MACOSX
	cb->macosxPrintCmd();
#endif
	}
	;

map	: STRING {cb->mapCmd($1);}
	| INT {cb->mapCmd($1);}
	;

set	: COLORMAP_ setColormap
	;

setColormap: STRING numeric numeric INT {cb->setColormapCmd($1, $2, $3, $4);}
	| INT numeric numeric INT {cb->setColormapCmd($1, $2, $3, $4);}
	| LEVEL_ {cnt=0;} levels {cb->setColormapLevelCmd(cnt,levels);}
	| RGB_ numeric numeric numeric numeric numeric numeric INT
	  {cb->setColormapCmd($2, $3, $4, $5, $6, $7, $8);}
	| WINDOW_ STRING {cb->setColormapWindowCmd($2);}
	;

levels	: levels level
	| level 
	;

level	: numeric {levels[cnt++] = $1;}
	;

postscript : COLORSPACE_ pscolorspace 
	  {cb->psColorSpaceCmd((Widget::PSColorSpace)$2);}
	| LEVEL_ INT {cb->psLevelCmd($2);}
	| RESOLUTION_ INT {cb->psResolutionCmd($2);}
	;

pscolorspace : BW_ {$$ = Widget::BW;}
	| GRAY_ {$$ = Widget::GRAY;}
	| RGB_ {$$ = Widget::RGB;}
	| CMYK_ {$$ = Widget::CMYK;}
	;

win32	: PRINT_ {
#ifdef _WIN32
	cb->win32PrintCmd();
#endif
	}
	;

%%
