// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __composite_h__
#define __composite_h__

#include "marker.h"
#include "list.h"

class Composite : public Marker {
 private:
  List<Marker> members;
  int global;

protected:
  virtual void renderX(Drawable, const Matrix&, RenderMode);
  virtual void renderPS(int);
#ifdef _MACOSX
  virtual void renderMACOSX();
#endif
#ifdef _WIN32
  virtual void renderWIN32();
#endif

public:
  Composite(const Composite&);
  Composite(FrameBase* p, const Vector& ctr, 
	    double ang, int gl,
	    const char* clr, int* dsh,
	    int wth, const char* fnt, const char* txt,
	    unsigned short prop, const char* cmt,
	    const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new Composite(*this);}

  void updateBBox();
  void updateCoords(const Matrix&);

  void draw(Drawable, int);
  void drawXOR();
  void drawMagnifier(const Matrix&, int);
  void ps(int,int);
#ifdef _MACOSX
  void macosx(int);
#endif
#ifdef _WIN32
  void win32(int);
#endif

  int isIn(const Vector& v);

  void setGlobal(int w) {global = w ? 1 : 0;}
  int getGlobal() {return global;}

  void list(ostream&, CoordSystem, SkyFrame, SkyFormat, int, int);
  void listXML(ostream&, CoordSystem, SkyFrame, SkyFormat) {}
  void listCiao(ostream&, CoordSystem, int, int);
  void listSAOtng(ostream&, CoordSystem, SkyFrame, SkyFormat, int);
  void listSAOimage(ostream&, int);
  void listPros(ostream&, CoordSystem, SkyFrame, SkyFormat, int);
  void listXY(ostream&, CoordSystem, SkyFrame, SkyFormat, int);

  void append(Marker*);
  Marker* extract();
};

#endif
