// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "fitsimage.h"

#include "NaN.h"
#include "smooth.h"

int FitsImage::smooth()
{
  // radius
  int r = parent->smoothRadius_;

  // src
  float* src = new float[width_*height_];
  if (!src)
    return 0;

  float* ptr = src;
  for (long jj=0; jj<height_; jj++)
    for (long ii=0; ii<width_; ii++, ptr++)
      *ptr = basedata_->getValueDouble(jj*width_+ii);

  // dest
  float* dest = (float*)smooth_->data();

  // kernel
  // create kernel
  int rr = 2*r+1;
  float* kernel = new float[rr*rr];
  if (!kernel)
    return 0;
  memset(kernel, 0, rr*rr*sizeof(float));

  switch (smoothFunction_) {
  case BOXCAR:
    boxcar(kernel,r);
    break;
  case TOPHAT:
    tophat(kernel,r);
    break;
  case GAUSSIAN:
    gaussian(kernel,r);
    break;
  }

  // convolve
  convolve(kernel,src,dest,width_,height_,r);
  
  // clean up
  delete [] kernel;
  delete [] src;

  return 1;
}
