// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "tcl.h"

#include "frame3dbase.h"
#include "fitsimage.h"

Frame3dBase::Frame3dBase(Tcl_Interp* i, Tk_Canvas c, Tk_Item* item) 
  : Base(i, c, item)
{
  zbufWidget_ = NULL;
  mkzbufWidget_ = NULL;
  zbufPanner_ = NULL;
  mkzbufPanner_ = NULL;
  zbufPS_ = NULL;
  mkzbufPS_ = NULL;

  zbufDepth_ =64;

  az_ =0;
  el_ =0;
  renderMethod_ = AIP;
  threshold_ =0;
}

Frame3dBase::~Frame3dBase()
{
  if (zbufWidget_)
    delete [] zbufWidget_;
  if (mkzbufWidget_)
    delete [] mkzbufWidget_;

  if (zbufPanner_)
    delete [] zbufPanner_;
  if (mkzbufPanner_)
    delete [] mkzbufPanner_;

  if (zbufPS_)
    delete [] zbufPS_;
  if (mkzbufPS_)
    delete [] mkzbufPS_;
}

double Frame3dBase::calcZoom(Vector src, Vector dest)
{
  return 1;
}

double Frame3dBase::calcPannerZoom()
{
  if (keyFits_ && *keyFits_ && pannerPixmap) {
    Vector3d ss = imageSize();
    Vector3d vrp = imageCenter() * imageToData3d;

    Vector3d llf = Vector3d(0,0,0);
    Vector3d lrf = Vector3d(0,ss[1],0);
    Vector3d urf = Vector3d(ss[0],ss[1],0);
    Vector3d ulf = Vector3d(ss[0],0,0);
 
    Vector3d llb = Vector3d(0,0,ss[2]);
    Vector3d lrb = Vector3d(0,ss[1],ss[2]);
    Vector3d urb = Vector3d(ss[0],ss[1],ss[2]);
    Vector3d ulb = Vector3d(ss[0],0,ss[2]);
 
    BBox3d bb;

    // 0, 0
    Matrix3d m0000 = 
      Translate3d(-vrp) *
      RotateY3d(-degToRad(0)) * 
      RotateX3d(degToRad(0)) * 
      RotateZ3d(0);
    bb.bound(llf*m0000);
    bb.bound(llf*m0000);
    bb.bound(lrf*m0000);
    bb.bound(urf*m0000);
    bb.bound(ulf*m0000);
    bb.bound(llb*m0000);
    bb.bound(lrb*m0000);
    bb.bound(urb*m0000);
    bb.bound(ulb*m0000);

    // 0, 90
    Matrix3d m0090 = 
      Translate3d(-vrp) *
      RotateY3d(-degToRad(90)) * 
      RotateX3d(degToRad(0)) * 
      RotateZ3d(0);
    bb.bound(llf*m0090);
    bb.bound(llf*m0090);
    bb.bound(lrf*m0090);
    bb.bound(urf*m0090);
    bb.bound(ulf*m0090);
    bb.bound(llb*m0090);
    bb.bound(lrb*m0090);
    bb.bound(urb*m0090);
    bb.bound(ulb*m0090);

    // 90, 0
    Matrix3d m9000 = 
      Translate3d(-vrp) *
      RotateY3d(-degToRad(0)) * 
      RotateX3d(degToRad(90)) * 
      RotateZ3d(0);
    bb.bound(llf*m9000);
    bb.bound(llf*m9000);
    bb.bound(lrf*m9000);
    bb.bound(urf*m9000);
    bb.bound(ulf*m9000);
    bb.bound(llb*m9000);
    bb.bound(lrb*m9000);
    bb.bound(urb*m9000);
    bb.bound(ulb*m9000);

    // 45, 45
    Matrix3d m4545 = 
      Translate3d(-vrp) *
      RotateY3d(-degToRad(45)) * 
      RotateX3d(degToRad(45)) * 
      RotateZ3d(0);
    bb.bound(llf*m4545);
    bb.bound(llf*m4545);
    bb.bound(lrf*m4545);
    bb.bound(urf*m4545);
    bb.bound(ulf*m4545);
    bb.bound(llb*m4545);
    bb.bound(lrb*m4545);
    bb.bound(urb*m4545);
    bb.bound(ulb*m4545);
 
    // 45, 90
    Matrix3d m4590 = 
      Translate3d(-vrp) *
      RotateY3d(-degToRad(90)) * 
      RotateX3d(degToRad(45)) * 
      RotateZ3d(0);
    bb.bound(llf*m4590);
    bb.bound(lrf*m4590);
    bb.bound(urf*m4590);
    bb.bound(ulf*m4590);
    bb.bound(llb*m4590);
    bb.bound(lrb*m4590);
    bb.bound(urb*m4590);
    bb.bound(ulb*m4590);

    // 90, 45
    Matrix3d m9045 = 
      Translate3d(-vrp) *
      RotateY3d(-degToRad(45)) * 
      RotateX3d(degToRad(90)) * 
      RotateZ3d(0);
    bb.bound(llf*m9045);
    bb.bound(lrf*m9045);
    bb.bound(urf*m9045);
    bb.bound(ulf*m9045);
    bb.bound(llb*m9045);
    bb.bound(lrb*m9045);
    bb.bound(urb*m9045);
    bb.bound(ulb*m9045);

    Vector3d bs = bb.size();
    double ll = bs[0]>bs[1] ? bs[0]:bs[1];
    return 1/ll*pannerWidth;
  }
  else
    return 1;
}

void Frame3dBase::centerImage()
{
  // Note: cursor is in USER coords, imageCenter() in IMAGE coords
  //   always center to center of pixel
  //   we have a rounding issue, so bump it before the floor()

  //  cursor = imageCenter().floor() + Vector(.5,.5);
  Vector3d center = imageCenter() * imageToData3d * 
    Translate3d(FLT_EPSILON,FLT_EPSILON,FLT_EPSILON);
  cursor = center.floor() * Translate3d(.5,.5,.5);
}

Vector3d Frame3dBase::imageCenter()
{
  return imageSize()/2. * dataToImage3d;
}

void Frame3dBase::pannerPush()
{
  // always render (to update panner background color)

  if (usePanner) {
    ostringstream str;

    str << pannerName << " update " << pannerPixmap << ";";

    // calculate bbox
    Vector ll = Vector(0,0) * widgetToPanner;
    Vector lr = Vector(options->width,0) * widgetToPanner;
    Vector ur = Vector(options->width,options->height) * widgetToPanner;
    Vector ul = Vector(0,options->height) * widgetToPanner;

    str << pannerName << " update bbox " << ll << lr << ur << ul << ";";

    // calculate image compass vectors
    Matrix3d mm = 
      RotateY3d(-az_) * 
      RotateX3d(el_) * 
      RotateZ3d(0);

    Vector xx = Vector((Vector3d(1,0,0)*mm).normalize())*FlipY();
    Vector yy = Vector((Vector3d(0,1,0)*mm).normalize())*FlipY();
    Vector zz = Vector((Vector3d(0,0,1)*mm).normalize())*FlipY();

    str << pannerName << " update image compass " << xx << yy << zz << ";";

    if (hasWCS(pannerSystem)) {
      Vector orval=currentFits_->pix2wcs(currentFits_->getWCScrpix(pannerSystem),
					pannerSystem, pannerSky);
      Vector orpix = currentFits_->wcs2pix(orval,pannerSystem,pannerSky);
      Vector origin = orpix;
      Vector delta = currentFits_->getWCScdelt(pannerSystem).abs();

      // find normalized north
      Vector north = currentFits_->wcs2pix(Vector(orval[0],orval[1]+delta[1]), 
					  pannerSystem,pannerSky);
      Vector nn = Vector((Vector3d(north-origin)*mm).normalize())*FlipY();

      // find normalized east
      Vector east = currentFits_->wcs2pix(Vector(orval[0]+delta[0],orval[1]), 
					 pannerSystem,pannerSky);
      Vector ee = Vector((Vector3d(east-origin)*mm).normalize())*FlipY();

      // and update the panner
      str << pannerName << " update wcs compass " << nn << ee << ends;
    }
    else
      str << pannerName << " update wcs compass invalid" << ends;

    Tcl_Eval(interp, str.str().c_str());
  }
}

int Frame3dBase::postscriptProc(int prepass)
{
  if (!visible)
    return TCL_OK;

  if (prepass)
    return TCL_OK;

  ps(&FitsImage::getmPSToData3d);

  return TCL_OK;
}

Matrix3d Frame3dBase::psMatrix(float scale, int width, int height)
{
  Vector zz = zoom_ * scale;
  return Scale3d(zz[0], zz[1], zbufDepth_ * 1/imageSize().length()) *
    Translate3d((int)(width/2.), 
		(int)(height/2.),
		(int)(zbufDepth_/2.));
}

void Frame3dBase::renderBBox(Pixmap pixmap, const Matrix3d& mm)
{
  Vector3d bbox = imageSize();

  Vector3d llf (0, 0, 0);
  Vector3d lrf (bbox[0], 0, 0);
  Vector3d urf (bbox[0],bbox[1],0);
  Vector3d ulf (0,bbox[1],0);

  Vector3d llb (0, 0, bbox[2]);
  Vector3d lrb (bbox[0], 0, bbox[2]);
  Vector3d urb (bbox[0],bbox[1],bbox[2]);
  Vector3d ulb (0,bbox[1],bbox[2]);

  Vector rllf = llf * mm;
  Vector rlrf = lrf * mm;
  Vector rurf = urf * mm;
  Vector rulf = ulf * mm;
  Vector rllb = llb * mm;
  Vector rlrb = lrb * mm;
  Vector rurb = urb * mm;
  Vector rulb = ulb * mm;

  XSetForeground(display, markerGC, getColor("black"));

  // front
  XDrawLine(display, pixmap, markerGC, rllf[0], rllf[1], rlrf[0], rlrf[1]);
  XDrawLine(display, pixmap, markerGC, rlrf[0], rlrf[1], rurf[0], rurf[1]);
  XDrawLine(display, pixmap, markerGC, rurf[0], rurf[1], rulf[0], rulf[1]);
  XDrawLine(display, pixmap, markerGC, rulf[0], rulf[1], rllf[0], rllf[1]);

  // back
  XDrawLine(display, pixmap, markerGC, rllb[0], rllb[1], rlrb[0], rlrb[1]);
  XDrawLine(display, pixmap, markerGC, rlrb[0], rlrb[1], rurb[0], rurb[1]);
  XDrawLine(display, pixmap, markerGC, rurb[0], rurb[1], rulb[0], rulb[1]);
  XDrawLine(display, pixmap, markerGC, rulb[0], rulb[1], rllb[0], rllb[1]);

  // other
  XDrawLine(display, pixmap, markerGC, rllf[0], rllf[1], rllb[0], rllb[1]);
  XDrawLine(display, pixmap, markerGC, rlrf[0], rlrf[1], rlrb[0], rlrb[1]);
  XDrawLine(display, pixmap, markerGC, rurf[0], rurf[1], rurb[0], rurb[1]);
  XDrawLine(display, pixmap, markerGC, rulf[0], rulf[1], rulb[0], rulb[1]);

  // text
  /*
  Tk_Font font = Tk_GetFont(interp, tkwin, "helvetica 10 normal");
  if (font)
    XSetFont(display, markerGC, Tk_FontId(font));

  XDrawString(display, pixmap, markerGC, rlrf[0], rlrf[1], "x", 1);
  XDrawString(display, pixmap, markerGC, rulf[0], rulf[1], "y", 1);
  XDrawString(display, pixmap, markerGC, rllb[0], rllb[1], "z", 1);
  */
}

void Frame3dBase::renderMagnifierCursor(const Vector& v) 
{
}

void Frame3dBase::reset()
{
  crosshair = cursor;
  centerImage();
  Base::reset();
}

void Frame3dBase::updateBase()
{
  if (DebugPerf)
    cerr << "updateBase...";

  Base::updateBase();

  int& width = options->width;
  int& height = options->height;

  if (doRender())
    ximageToPixmap(basePixmap, baseXImage, &FitsImage::getmWidgetToData3d);
  else {
    XSetForeground(display, gc, bgColor->pixel);
    XFillRectangle(display, basePixmap, gc, 0, 0, width, height);
  }

  // panner
  updatePanner();

  if (DebugPerf)
    cerr << " end" << endl;
}

void Frame3dBase::updateMagnifierMatrix()
{
  // calculate partial magnifier matrix
  // This is a partial matix, it lacks the Translate at the begining

  userToMagnifier =                     
    Scale(zoom_) *                      // scale about cursor position
    Scale(magnifierZoom_) *             // scale
    Translate(magnifierWidth/2., magnifierHeight/2.);

  userToMagnifier3d =
    Scale3d(zoom_[0], zoom_[1], zbufDepth_ * 1/imageSize().length()) *
    Scale3d(magnifierZoom_,magnifierZoom_,1) *
    Translate3d((int)(magnifierWidth/2.), 
		(int)(magnifierHeight/2.), 
		(int)(zbufDepth_/2.));
}

void Frame3dBase::updateMagnifierZoom(const Vector& v)
{
    // v is in CANVAS coords
  Matrix3d refToMagnifier3d = refToUser3d * Translate3d(-(v * canvasToUser)) * 
    userToMagnifier3d;
  Matrix3d magnifierToRef3d = refToMagnifier3d.invert();

  updateMagnifierMatrices(refToMagnifier3d);

  // Update ximage
  buildXImage(magnifierXImage, &FitsImage::getmMagnifierToData);

  // copy XImage to Pixmap
  XPutImage(display, magnifierPixmap, gc, magnifierXImage, 0, 0, 0, 0, 
	    magnifierWidth, magnifierHeight);

  if (useMagnifierGraphics) {
  }
}

void Frame3dBase::updateMatrices()
{
  if (DebugPerf)
    cerr << "updateMatrices..." << endl;

  // refToUser
  refToUser = FlipY();
  userToRef = refToUser.invert();

  // userToWidget
  userToWidget = 
    Scale(zoom_) *
    Translate((int)(options->width/2.), (int)(options->height/2.));
  widgetToUser = userToWidget.invert();

  Base::updateMatrices();

  // Panner Matrix
  userToPanner =
    Scale(calcPannerZoom()) *
    Translate(pannerWidth/2., pannerHeight/2.);
  pannerToUser = userToPanner.invert();

  refToPanner = refToUser * userToPanner;
  pannerToRef = refToPanner.invert();

  pannerToWidget = pannerToUser * userToWidget;
  widgetToPanner = pannerToWidget.invert();

  // we want a parallel projection, with no shear, no scale
  // view reference point
  // Note: imageCenter() is in IMAGE coords
  Vector3d vrp = imageCenter() * imageToData3d;

  threedToRef3d = 
    Translate3d(-vrp) *
    RotateY3d(-az_) * 
    RotateX3d(el_) * 
    RotateZ3d(0);

  // refToUser
  refToUser3d = Scale3d(1,-1,1);
  userToRef3d = refToUser3d.invert();

  // userToWidget
  userToWidget3d = 
    Scale3d(zoom_[0], zoom_[1], zbufDepth_ * 1/imageSize().length()) *
    Translate3d((int)(options->width/2.), 
		(int)(options->height/2.), 
		(int)(zbufDepth_/2.));
  widgetToUser3d = userToWidget3d.invert();

  refToWidget3d = refToUser3d * userToWidget3d;
  widgetToRef3d = refToWidget3d.invert();

  // Panner Matrix
  double zz = calcPannerZoom();
  userToPanner3d =
    Scale3d(zz,zz,zbufDepth_ * 1/imageSize().length()) *
    Translate3d(pannerWidth/2.,pannerHeight/2.,zbufDepth_/2.);
  pannerToUser3d = userToPanner3d.invert();

  // Magnifier Matrix
  updateMagnifierMatrix();

  // delete current zbuffer since matrices have changed
  if (zbufWidget_)
    delete [] zbufWidget_;
  zbufWidget_ = NULL;
  if (mkzbufWidget_)
    delete [] mkzbufWidget_;
  mkzbufWidget_ = NULL;

  if (zbufPanner_)
    delete [] zbufPanner_;
  zbufPanner_ = NULL;
  if (mkzbufPanner_)
    delete [] mkzbufPanner_;
  mkzbufPanner_ = NULL;

  if (zbufPS_)
    delete [] zbufPS_;
  zbufPS_ = NULL;
  if (mkzbufPS_)
    delete [] mkzbufPS_;
  mkzbufPS_ = NULL;

  if (DebugPerf)
    cerr << "updateMatrices end" << endl;
}

void Frame3dBase::updatePanner()
{
  if (usePanner)
    if (doRender()) {
      ximageToPixmap(pannerPixmap, pannerXImage,&FitsImage::getmPannerToData3d);
      renderBBox(pannerPixmap, (*keyFits_)->getDataToPanner3d());
    }
    else {
      XSetForeground(display, gc, bgColor->pixel);
      XFillRectangle(display, pannerPixmap, gc, 0, 0, pannerWidth,pannerHeight);
    }  

  pannerPush();
}

void Frame3dBase::updatePM(const BBox& bbox)
{
  // bbox is in Canvas Coords

  if (DebugPerf)
    cerr << "updatePM..." << endl;

  Base::updatePM(bbox);

  // hightlite bbox
  renderHighLite();

  if (DebugPerf)
    cerr << "updatePM end" << endl;
}

#ifdef _MACOSX
void Frame3dBase::macosxPrintCmd()
{
  if (!visible)
    return;

  // init
  macosxBegin();

  // image
  macosx(&FitsImage::getmPSToData3d);

  // cleanup
  macosxEnd();
}
#endif

#ifdef _WIN32
void Frame3dBase::win32PrintCmd()
{
  if (!visible)
    return;

  // init
  win32Begin();

  // image
  win32(&FitsImage::getmPSToData3d);

  // cleanup
  win32End();
}
#endif
