// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __frame3dbase_h__
#define __frame3dbase_h__

#include "base.h"

class Frame3dBase : public Base {
 protected:
  Vector3d cursor; // current cursor position in USER coords
  Vector3d crosshair; // crosshair location in REF coords

  float* zbufWidget_;
  unsigned char* mkzbufWidget_;
  float* zbufPanner_;
  unsigned char* mkzbufPanner_;
  float* zbufPS_;
  unsigned char* mkzbufPS_;

  int zbufDepth_;

  Matrix3d threedToRef3d;

  Matrix3d pannerToRef3d;
  Matrix3d pannerToUser3d;
  Matrix3d pannerToWidget3d;
  Matrix3d refToPanner3d;
  Matrix3d refToUser3d;
  Matrix3d refToWidget3d;
  Matrix3d userToMagnifier3d;
  Matrix3d userToPanner3d;
  Matrix3d userToRef3d;
  Matrix3d userToWidget3d;
  Matrix3d widgetToPanner3d;
  Matrix3d widgetToUser3d;
  Matrix3d widgetToRef3d;

  float az_;
  float el_;
  RenderMethod renderMethod_;
  float threshold_;

 private:
  double calcPannerZoom();
  void pannerPush();
  void resetValues() {}
  void renderMagnifierCursor(const Vector& v);
  void updateBase();
  void updateMagnifierMatrix();
  void updateMagnifierZoom(const Vector& v);
  virtual void updateMagnifierMatrices(Matrix3d&) =0;
  void updatePM(const BBox&);

 protected:
  double calcZoom(Vector,Vector);
  void centerImage();
  Vector3d imageCenter();
  virtual Vector3d imageSize() =0;
  int postscriptProc(int prepass);
  Matrix3d psMatrix(float scale, int width, int height);
  void reset();
  void renderBBox(Pixmap, const Matrix3d&);
  void updateMatrices();
  void updatePanner();

 public:
  Frame3dBase(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~Frame3dBase();

  // Marker Commands
  void hasMarkerHighlitedCmd();
  void hasMarkerPasteCmd();
  void hasMarkerSelectedCmd();
  void hasMarkerUndoCmd();

  // Print Functions
#ifdef _MACOSX
  void macosxPrintCmd();
#endif
#ifdef _WIN32
  void win32PrintCmd();
#endif

  // 3d Commands
  void get3dRenderMethodCmd();
  void get3dThresholdCmd();
  void get3dViewCmd();
  void set3dRenderMethodCmd(RenderMethod);
  void set3dThresholdCmd(float);
  void set3dViewCmd(float, float);
};

#endif
