// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "tcl.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "frame3dtruecolor.h"
#include "fitsimage.h"
#include "ps.h"
#include "NaN.h"

Frame3dTrueColor::Frame3dTrueColor(Tcl_Interp* i, Tk_Canvas c, Tk_Item* item)
  : Frame3dBase(i,c,item), Frame3d(i,c,item), Frame3dTrue(i,c,item)
{
  colormapData = NULL;
}

Frame3dTrueColor::~Frame3dTrueColor()
{
  if (colormapData)
    delete [] colormapData;
}

void Frame3dTrueColor::buildXImage(XImage* ximage,
				   double* (FitsImage::*getMatrix)())
{
  // we need a colorScale before we can render
  if (!validColorScale())
    return;

  unsigned char* img = fillImage(ximage->width, ximage->height, getMatrix);
  if (img)
    encodeTrueColor(img, ximage);
  delete [] img;
}

// Commands

void Frame3dTrueColor::colormapCmd(int id, float b, float c, int i, 
				 unsigned short* index, unsigned char* cells, 
				 int cnt)
{
  // first check for change
  if (cmapID == id && bias == b && contrast == c && invert == i && colorCells)
    return;

  // we got a change
  cmapID = id;
  bias = b;
  contrast = c;
  invert = i;

  updateColorCells(index, cells, cnt);
  updateColorScale();

  if (mosaicCount) {
    update(BASE); // always update
    skipUpdate = 0;
  }
  else {
    if (!skipUpdate) {
      update(BASE); // update once
      skipUpdate = 1;
    }      
  }
}

void Frame3dTrueColor::colormapEndCmd()
{
  update(BASE);
}
