// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

%pure-parser
%parse-param {Base* fr}
%lex-param {frFlexLexer* ll}
%parse-param {frFlexLexer* ll}

%{
#define YYDEBUG 1

#include <math.h>
#include <string.h>
#include <iostream>

#include "base.h"
#include "frame.h"
#include "framergb.h"
#include "fitsimage.h"
#include "vector.h"
#include "coord.h"
#include "fvcontour.h"

#undef yyFlexLexer
#define yyFlexLexer frFlexLexer
#include <FlexLexer.h>

extern int frlex(void*, frFlexLexer*);
extern void frerror(Base*, frFlexLexer*, const char*);

static CoordSystem currentCoord = IMAGE;
static SkyFrame currentSky = FK5;

static unsigned short defaultProps = Marker::SELECT | Marker::HIGHLITE |
		Marker::EDIT | Marker::MOVE | Marker::ROTATE | 
		Marker::DELETE | Marker::INCLUDE | Marker::SOURCE;
static unsigned short currentProps;
static char currentColor[16];
static int currentWidth;
static int currentDash[2];
static char currentFont[32];
static char currentText[80];

static List<Tag> taglist;
static List<CallBack> cblist;

static unsigned short propQMask;
static unsigned short propQValue;

static void setProps(unsigned short* props, unsigned short prop, int value);
%}

%union {
  char chr;
  char str[2048];
  void* ptr;
  int integer;
  double real;
  double vector[3];
  int dash[2];
}

%type <real> numeric
%type <integer> yesno

%type <real> angle
%type <real> optangle
%type <integer> precision
%type <real> sexagesimal
%type <real> hms
%type <real> dms
%type <vector> coord
%type <integer> coordSystem
%type <integer> wcsSystem
%type <integer> internalSystem
%type <integer> skyFrame
%type <integer> skyFormat

%type <integer> markerProperty
%type <integer> markerCallBack
%type <integer> markerFormat
%type <integer> markerLayer
%type <dash> markerDash
%type <integer> pscolorspace
%type <integer> scaleType
%type <integer> shmType
%type <integer> contourmethod
%type <integer> incrLoad
%type <integer> layerType
%type <integer> gridType
%type <integer> compressType
%type <integer> fileNameType
%type <integer> smoothFunction
%type <integer> pointSize
%type <integer> endian
%type <integer> renderMethod

%token <real> REAL
%token <integer> INT
%token <str> STRING
%token <ptr> POINTER

%token <integer> HOUR
%token <integer> MINUTE
%token <real> SECOND
%token <real> ANGDEGREE
%token <real> ARCMINUTE
%token <real> ARCSECOND
%token <real> ANGRADIAN

%token <str> SEXSTR
%token <str> HMSSTR
%token <str> DMSSTR

%token ABOUT_
%token AIP_
%token ALLOC_
%token ALLOCGZ_
%token ALIGN_
%token ALL_
%token AMPLIFIER_
%token ANALYSIS_
%token ANGLE_
%token ANNULUS_
%token APPEND_
%token ARCMIN_
%token ARCSEC_
%token ARRAY_
%token ARROW_
%token AST_
%token AUTO_
%token AVERAGE_
%token B1950_
%token BACK_
%token BASE_
%token BBOX_
%token BEGIN_
%token BG_
%token BIGENDIAN_
%token BIN_
%token BITPIX_
%token BLEND_
%token BLOCK_
%token BOX_
%token BOXANNULUS_
%token BOXCAR_
%token BOXCIRCLE_
%token BPANDA_
%token BUFFER_
%token BW_
%token CALLBACK_
%token CANVAS_
%token CATALOG_
%token CENTER_
%token CENTROID_
%token CHANNEL_
%token CIRCLE_
%token CIAO_
%token CLEAR_
%token CLIP_
%token COLORMAP_
%token COLOR_
%token COLORSCALE_
%token COLORSPACE_
%token COLS_
%token COMMAND_
%token COMPASS_
%token COMPOSITE_
%token COMPRESS_
%token CONTOUR_
%token CONTRAST_
%token COORDINATES_
%token COPY_
%token COUNT_
%token CPANDA_
%token CREATE_
%token CROSS_
%token CROSSHAIR_
%token CUBE_
%token CURSOR_
%token CUT_
%token CMYK_
%token DASH_
%token DASHLIST_
%token DATA_
%token DATAMIN_
%token DATASEC_
%token DEBUG_
%token DEGREES_
%token DEFAULT_
%token DELETE_
%token DEPTH_
%token DETECTOR_
%token DIAMOND_
%token DIM_
%token DS9_
%token EDIT_
%token ECLIPTIC_
%token ELLIPSE_
%token ELLIPSEANNULUS_
%token END_
%token EPANDA_
%token EQUATORIAL_
%token ERASE_
%token EXT_
%token FACTOR_
%token FALSE_
%token FILE_
%token FILTER_
%token FIT_
%token FITS_
%token FITSY_
%token FIXED_
%token FK4_
%token FK5_
%token FONT_
%token FRONT_
%token FULL_
%token FUNCTION_
%token GALACTIC_
%token GAUSSIAN_
%token GET_
%token GLOBAL_
%token GRAPHICS_
%token GRAY_
%token GRID_
%token GZ_
%token HANDLE_
%token HAS_
%token HEADER_
%token HEIGHT_
%token HIDE_
%token HIGH_
%token HIGHLITE_
%token HISTEQU_
%token HISTOGRAM_
%token HMS_
%token HORIZONTAL_
%token ICRS_
%token ID_
%token IIS_
%token IMAGE_
%token INCLUDE_
%token INCR_
%token INFO_
%token INTEGER_
%token IRAF_
%token IRAFMIN_
%token J2000_
%token JPEG_
%token KEY_
%token KEYWORD_
%token LABEL_
%token LENGTH_
%token LEVEL_
%token LITTLEENDIAN_
%token LINE_
%token LINEAR_
%token LIST_
%token LOAD_
%token LOCAL_
%token LOG_
%token LOW_
%token MACOSX_
%token MAGNIFIER_
%token MAP_
%token MARK_
%token MARKER_
%token MASK_
%token MESSAGE_
%token METHOD_
%token MINMAX_
%token MIP_
%token MMAP_
%token MMAPINCR_
%token MOSAIC_
%token MODE_
%token MOTION_
%token MOVE_
%token NAME_
%token NAN_
%token NEW_
%token NEXT_
%token NO_
%token NONE_
%token NOW_
%token NUMBER_
%token OBJECT_
%token OFF_
%token ON_
%token ONLY_
%token OPTION_
%token ORIENT_
%token PAN_
%token PANNER_
%token PARSER_
%token PASTE_
%token PERF_
%token PHYSICAL_
%token PIXEL_
%token PNG_
%token POINT_
%token POINTER_
%token POLYGON_
%token POSTSCRIPT_
%token POW_
%token PPM_
%token PRINT_
%token PRESERVE_
%token PROJECTION_
%token PROPERTY_
%token PUBLICATION_
%token PROS_
%token RADIUS_
%token REGION_
%token REPLACE_
%token RESAMPLE_
%token RESET_
%token RESOLUTION_
%token RGB_
%token ROOT_
%token ROTATE_
%token RULER_
%token SAMPLE_
%token SAOIMAGE_
%token SAOTNG_
%token SAVE_
%token SCAN_
%token SCIENTIFIC_
%token SCOPE_
%token SEGMENT_
%token SELECT_
%token SET_
%token SEXAGESIMAL_
%token SHARED_
%token SHIFT_
%token SHMID_
%token SHOW_
%token SIZE_
%token SKY_
%token SLICE_
%token SMMAP_
%token SMOOTH_
%token SOCKET_
%token SOCKETGZ_
%token SOURCE_
%token SQRT_
%token SQUARED_
%token SSHARED_
%token STATUS_
%token SUM_
%token SYSTEM_
%token TABLE_
%token TAG_
%token TEMPLATE_
%token TEXT_
%token THREED_
%token THRESHOLD_
%token THICK_
%token TIFF_
%token TRANSPARENCY_
%token TO_
%token TOGGLE_
%token TOPHAT_
%token TRUE_
%token TYPE_
%token UNDO_
%token UNHIGHLITE_
%token UNLOAD_
%token UNSELECT_
%token UPDATE_
%token USER_
%token VALUE_
%token VAR_
%token VIEW_
%token VECTOR_
%token VERSION_
%token VERTEX_
%token VERTICAL_
%token WARP_
%token WCS_
%token WCSA_
%token WCSB_
%token WCSC_
%token WCSD_
%token WCSE_
%token WCSF_
%token WCSG_
%token WCSH_
%token WCSI_
%token WCSJ_
%token WCSK_
%token WCSL_
%token WCSM_
%token WCSN_
%token WCSO_
%token WCSP_
%token WCSQ_
%token WCSR_
%token WCSS_
%token WCST_
%token WCSU_
%token WCSV_
%token WCSW_
%token WCSX_
%token WCSY_
%token WCSZ_
%token WCS0_
%token WFPC2_
%token WIDTH_
%token WIN32_
%token XML_
%token XY_
%token YES_
%token ZMAX_
%token ZSCALE_
%token ZOOM_

%%

command : DEBUG_ debug

	| BIN_ bin
	| BG_ COLOR_ STRING {fr->bgColorCmd($3);}
	| CENTER_ {fr->centerCmd();}
	| CLEAR_ {fr->clearCmd();}
	| CLIP_ clip
	| COLORMAP_ colormap
	| COLORSCALE_ scaleType 
	  {fr->colorScaleCmd((FrScale::ColorScaleType)$2);}
	| CONTOUR_ contour
	| CROSSHAIR_ crosshair
	| DATASEC_ yesno {fr->DATASECCmd($2);}
	| FITSY_ fitsy
	| GET_ get
	| GRID_ grid
	| HAS_ has
	| HIDE_ {fr->hideCmd();}
	| HIGHLITE_ yesno {fr->highliteCmd($2);}
	| IIS_ iis
	| LOAD_ load
	| MACOSX_ macosx
	| MAGNIFIER_ magnifier
	| MARKER_ markerLayer marker
	| MASK_ mask
	| NAN_ COLOR_ STRING {fr->nanColorCmd($3);}
	| ORIENT_ orient
	| PAN_ pan
	| PANNER_ panner
	| POSTSCRIPT_ postscript
	| RESET_ {fr->resetCmd();}
	| REGION_ markerLayer region
	| RGB_ rgb
	| ROTATE_ rotate
	| SAVE_ save
	| SHOW_ {fr->showCmd();}
	| SMOOTH_ smooth
	| THREED_ threed
	| UNLOAD_ {fr->unloadFitsCmd();}
	| UPDATE_ update
	| VERSION_ {fr->msg("Frame 1.0");}
	| WARP_ warp
	| WCS_ wcs
	| WIN32_ win32
	| ZOOM_ zoom
	;

numeric	: REAL {$$=$1;}
	| INT {$$=$1;}
	;

debug	: yesno {yydebug=$1;}
	| AST_ yesno {DebugAST=$2;}
	| MOSAIC_ yesno {DebugMosaic=$2;}
	| PARSER_ yesno {yydebug=$2;}
	| PERF_ yesno {DebugPerf=$2;}
	| WCS_ yesno {DebugWCS=$2;}
	| BIN_ yesno {DebugBin=$2;}
	| COMPRESS_ yesno {DebugCompress=$2}
	| GZ_ yesno {DebugGZ=$2;}
	| RGB_ yesno {DebugRGB=$2;}
	;

yesno	: INT {$$=($1 ? 1 : 0);}

	| YES_ {$$=1;}
	| 'Y' {$$=1;}
	| ON_ {$$=1;}
	| TRUE_ {$$=1;}

	| NO_ {$$=0;}
	| 'N' {$$=0;}
	| OFF_ {$$=0;}
	| FALSE_ {$$=0;}
	;

precision : /* empty */ {$$ = DEFAULT;}
	| DEFAULT_ {$$ = DEFAULT;}
	| FIXED_ {$$ = FIXED;}
	| SCIENTIFIC_ {$$ = SCIENTIFIC;}
	| INTEGER_ {$$ = INTEGER;}
	;

fileNameType : /* empty */ {$$ = Base::ROOTBASE;}
	| ROOT_ BASE_ {$$ = Base::ROOTBASE;}
	| FULL_ BASE_ {$$ = Base::FULLBASE;}
	| ROOT_ {$$ = Base::ROOT;}
	| FULL_ {$$ = Base::FULL;}
	;

optangle : /* empty */ {$$ = 0;}
	| angle {$$ = $1;}
	;

angle	: numeric {$$ = degToRad($1);} /* assume degree */
	| ANGDEGREE {$$ = degToRad($1);}
	| ANGRADIAN {$$=$1;}
	;

sexagesimal: SEXSTR {$$ = parseSEXStr($1);}
	;

hms	: HMSSTR {$$ = parseHMSStr($1);}
	;

dms	: DMSSTR {$$ = parseDMSStr($1);}
	;

coord	: sexagesimal sexagesimal
	{
	  Vector r;
	  if (currentSky == GALACTIC || currentSky == ECLIPTIC) 
	    r = Vector($1,$2);
	  else
	    r = Vector($1*360./24.,$2);

	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| hms dms
        {
	  Vector r = Vector($1,$2);
	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| dms dms
        {
	  Vector r = Vector($1,$2);
	  $$[0] = r[0];
	  $$[1] = r[1];
	  $$[2] = r[2];
	}
	| numeric numeric 
	{
	  $$[0] = $1;
	  $$[1] = $2;
	  $$[2] = 1;
	}
	;

coordSystem :IMAGE_ {$$ = currentCoord = IMAGE;}
	| PHYSICAL_ {$$ = currentCoord = PHYSICAL;}
	| DETECTOR_ {$$ = currentCoord = DETECTOR;}
	| AMPLIFIER_ {$$ = currentCoord = AMPLIFIER;}
	| wcsSystem {$$ = (CoordSystem)$1;}
	;

wcsSystem : WCS_ {$$ = currentCoord = WCS;}
	| WCSA_ {$$ = currentCoord = WCSA;}
	| WCSB_ {$$ = currentCoord = WCSB;}
	| WCSC_ {$$ = currentCoord = WCSC;}
	| WCSD_ {$$ = currentCoord = WCSD;}
	| WCSE_ {$$ = currentCoord = WCSE;}
	| WCSF_ {$$ = currentCoord = WCSF;}
	| WCSG_ {$$ = currentCoord = WCSG;}
	| WCSH_ {$$ = currentCoord = WCSH;}
	| WCSI_ {$$ = currentCoord = WCSI;}
	| WCSJ_ {$$ = currentCoord = WCSJ;}
	| WCSK_ {$$ = currentCoord = WCSK;}
	| WCSL_ {$$ = currentCoord = WCSL;}
	| WCSM_ {$$ = currentCoord = WCSM;}
	| WCSN_ {$$ = currentCoord = WCSN;}
	| WCSO_ {$$ = currentCoord = WCSO;}
	| WCSP_ {$$ = currentCoord = WCSP;}
	| WCSQ_ {$$ = currentCoord = WCSQ;}
	| WCSR_ {$$ = currentCoord = WCSR;}
	| WCSS_ {$$ = currentCoord = WCSS;}
	| WCST_ {$$ = currentCoord = WCST;}
	| WCSU_ {$$ = currentCoord = WCSU;}
	| WCSV_ {$$ = currentCoord = WCSV;}
	| WCSW_ {$$ = currentCoord = WCSW;}
	| WCSX_ {$$ = currentCoord = WCSX;}
	| WCSY_ {$$ = currentCoord = WCSY;}
	| WCSZ_ {$$ = currentCoord = WCSZ;}
	;

internalSystem : CANVAS_ {$$ = CANVAS;}
	| PANNER_ {$$ = PANNER;}
	;

skyFrame : /* empty */ {$$ = currentSky = FK5;}
	| FK4_ {$$ = currentSky = FK4;}
	| B1950_ {$$ = currentSky = FK4;}
	| FK5_ {$$ = currentSky = FK5;}
	| J2000_ {$$ = currentSky = FK5;}
	| ICRS_ {$$ = currentSky = ICRS;}
	| GALACTIC_ {$$ = currentSky = GALACTIC;}
	| ECLIPTIC_ {$$ = currentSky = ECLIPTIC;}
	;

skyFormat : /* empty */ {$$=DEGREES;}
	| DEGREES_ {$$=DEGREES;}
	| SEXAGESIMAL_ {$$=SEXAGESIMAL;}
	| HMS_ {$$=HMS;}
	| ARCMIN_ {$$=ARCMIN;}
	| ARCSEC_ {$$=ARCSEC;}
	;

scaleType: LINEAR_ {$$ = FrScale::LINEARSCALE;}
	| LOG_ {$$ = FrScale::LOGSCALE;}
	| POW_ {$$ = FrScale::POWSCALE;}
	| SQRT_ {$$ = FrScale::SQRTSCALE;}
	| SQUARED_ {$$ = FrScale::SQUAREDSCALE;}
	| HISTEQU_ {$$ = FrScale::HISTEQUSCALE;}
	;	

shmType	: /* empty */ {$$ = Base::SHMID;}
	| SHMID_ {$$ = Base::SHMID;}
	| KEY_ {$$ = Base::KEY;}
	;

incrLoad: /* empty */ {$$ = Base::LOADALL;}
	| ALL_ {$$ = Base::LOADALL;}
	| INCR_ {$$ = Base::INCR;}
	;

layerType : /* empty */ {$$ = Base::IMG;}
	| IMAGE_ {$$ = Base::IMG;}
	| MASK_ {$$ = Base::MASK;}
	;

compressType: /* empty */ {$$ = Base::NOCOMPRESS;}
	| NONE_ {$$ = Base::NOCOMPRESS;}
	| GZ_ {$$ = Base::GZ;}
	;

pointSize: /* empty */ {$$ = POINTSIZE;}
	| INT {$$ = $1;}
	;

endian	: /* empty */ {$$ = FitsFile::BIGENDIAN;}
	| BIGENDIAN_ {$$ = FitsFile::BIGENDIAN;}
	| LITTLEENDIAN_ {$$ = FitsFile::LITTLEENDIAN;}
	;

threed	: VIEW_ numeric numeric {fr->set3dViewCmd($2,$3);}
	| METHOD_ renderMethod 
	  {fr->set3dRenderMethodCmd((Base::RenderMethod)$2);}
	| THRESHOLD_ numeric {fr->set3dThresholdCmd($2);}
	;

bin	: ABOUT_ binAbout
	| COLS_ STRING STRING STRING {fr->binColsCmd($2,$3,$4);}
	| DEPTH_ INT {fr->binDepthCmd($2);}
	| FACTOR_ binFactor
	| FUNCTION_ binFunction
	| BUFFER_ SIZE_ INT {fr->binBufferSizeCmd($3);}
	| TO_ binTo
	| FILTER_ STRING {fr->binFilterCmd($2);}
	;

binAbout : CENTER_ {fr->binAboutCmd();}
	| numeric numeric {fr->binAboutCmd(Vector($1,$2));}
	;

binFactor : numeric {fr->binFactorCmd(Vector($1,$1));}
	| numeric numeric {fr->binFactorCmd(Vector($1,$2));}
	| numeric ABOUT_ numeric numeric
	  {fr->binFactorAboutCmd(Vector($1,$1), Vector($3,$4));}
	| numeric numeric ABOUT_ numeric numeric
	  {fr->binFactorAboutCmd(Vector($1,$2), Vector($4,$5));}
	| TO_ numeric {fr->binFactorToCmd(Vector($2,$2));}
	| TO_ numeric numeric {fr->binFactorToCmd(Vector($2,$3));}
	| TO_ numeric ABOUT_ numeric numeric
	  {fr->binFactorToAboutCmd(Vector($2,$2), Vector($4,$5));}
	| TO_ numeric numeric ABOUT_ numeric numeric
	  {fr->binFactorToAboutCmd(Vector($2,$3), Vector($5,$6));}
	;

binFunction : AVERAGE_ {fr->binFunctionCmd(FitsHist::AVERAGE);}
	| SUM_ {fr->binFunctionCmd(FitsHist::SUM);}
	;

binTo	: FIT_ {fr->binToFitCmd();}
	| numeric numeric ABOUT_ CENTER_ STRING STRING STRING 
	  {fr->binCmd(Vector($1,$2), $5, $6, $7);}
	| numeric numeric INT numeric numeric ABOUT_ CENTER_ 
	    STRING STRING STRING STRING
	  {fr->binCmd(Vector($1,$2), $3, Vector($4,$5), $8, $9, $10, $11);}
	| numeric numeric ABOUT_ numeric numeric STRING STRING STRING 
	  {fr->binCmd(Vector($1,$2), Vector($4,$5), $6, $7, $8);}
	| numeric numeric INT numeric numeric ABOUT_ numeric numeric 
	    STRING STRING STRING STRING
	  {fr->binCmd(Vector($1,$2), $3, Vector($4,$5), Vector($7,$8), 
	    $9, $10, $11, $12);}
	;

clip	: SCOPE_ clipScope
	| MODE_ clipMode
	| MINMAX_ clipMinMax
	| USER_ clipUser
	| ZSCALE_ clipZScale
	| PRESERVE_ yesno {fr->clipPreserveCmd($2);}
	;

clipUser: /* empty */ numeric numeric {fr->clipUserCmd($1,$2);}
	| LOW_ numeric {fr->clipUserLowCmd($2);} 
	| HIGH_ numeric {fr->clipUserHighCmd($2);} 
	;

clipScope: GLOBAL_ {fr->clipScopeCmd(FrScale::GLOBAL);}
	| LOCAL_ {fr->clipScopeCmd(FrScale::LOCAL);}
	;

clipMode: numeric {fr->clipModeCmd($1);}
	| MINMAX_ {fr->clipModeCmd(FrScale::MINMAX);}
	| ZSCALE_ {fr->clipModeCmd(FrScale::ZSCALE);}
	| ZMAX_ {fr->clipModeCmd(FrScale::ZMAX);}
	| USER_ {fr->clipModeCmd(FrScale::USERCLIP);}
	;

clipMinMax : MODE_ clipMinMaxMode
	| SAMPLE_ INT {fr->clipMinMaxSampleCmd($2);}
	;

clipMinMaxMode : AUTO_ {fr->clipMinMaxModeCmd(FrScale::AUTOSCAN);}
	| SCAN_ {fr->clipMinMaxModeCmd(FrScale::SCAN);}
	| SAMPLE_ {fr->clipMinMaxModeCmd(FrScale::SAMPLE);}
	| DATAMIN_ {fr->clipMinMaxModeCmd(FrScale::DATAMIN);}
	| IRAFMIN_ {fr->clipMinMaxModeCmd(FrScale::IRAFMIN);}
	;

clipZScale: CONTRAST_ numeric {fr->clipZScaleContrastCmd($2);}
        | SAMPLE_ INT {fr->clipZScaleSampleCmd($2);}
	| LINE_ INT {fr->clipZScaleLineCmd($2);}
	;

colormap : INT numeric numeric INT POINTER POINTER INT 
	  {fr->colormapCmd($1, $2, $3, $4, (unsigned short*)$5, 
	    (unsigned char*)$6, $7);}
	| RGB_ numeric numeric numeric numeric numeric numeric INT POINTER INT
	  {fr->colormapCmd($2,$3,$4,$5,$6,$7,$8,(unsigned char*)$9,$10);}
	| BEGIN_ {fr->colormapBeginCmd();}
	| MOTION_ colormapMotion
	| END_ {fr->colormapEndCmd();}
	;

colormapMotion: INT numeric numeric INT POINTER POINTER INT
	  {fr->colormapMotionCmd($1, $2, $3, $4, (unsigned short*)$5,
	    (unsigned char*)$6, $7);}
	| RGB_ numeric numeric numeric numeric numeric numeric INT POINTER INT
	  {fr->colormapMotionCmd($2,$3,$4,$5,$6,$7,$8,
	    (unsigned char*)$9,$10);}

contour	: COLOR_ STRING {fr->contourSetColorCmd($2);}
	| COPY_ coordSystem skyFrame 
	  {fr->contourCopyCmd((CoordSystem)$2, (SkyFrame)$3);}
	| CREATE_ contourCreate
	| DASH_ INT  {fr->contourSetDashCmd($2);}
	| DELETE_ {fr->contourDeleteCmd();}
	| DELETE_ ALL_ {fr->contourDeleteAllCmd();}
	| LEVEL_ STRING {fr->contourLevelCmd($2);}
	| LOAD_ STRING INT INT STRING coordSystem skyFrame
	  {fr->contourLoadCmd($2, $3, $4, $5, (CoordSystem)$6, (SkyFrame)$7);}
	| PASTE_ STRING INT INT POINTER coordSystem skyFrame
	  {fr->contourPasteCmd($2, $3, $4, $5, (CoordSystem)$6, (SkyFrame)$7);}
	| SAVE_ STRING coordSystem skyFrame
	  {fr->contourSaveCmd($2, (CoordSystem)$3, (SkyFrame)$4);}
	| WIDTH_ INT {fr->contourSetLineWidthCmd($2);}
	;

contourCreate : STRING INT INT INT numeric contourmethod 
	  {fr->contourCreateCmd($1,$2,$3,$4,$5,$6);}
	| STRING INT INT numeric contourmethod STRING
	  {fr->contourCreateCmd($1,$2,$3,$4,$5,$6);}
	| POLYGON_ markerProperties
	  {fr->createContourPolygonCmd(currentColor,currentDash,currentWidth, 
	    currentFont, currentText, currentProps, NULL, taglist,cblist);}
	;

contourmethod : SMOOTH_ {$$ = FVContour::SMOOTH;}
	| BLOCK_ {$$ = FVContour::BLOCK;}
	;

crosshair: BEGIN_ MOTION_ internalSystem numeric numeric
	  {fr->crosshairBeginCmd(Vector($4,$5), (InternalSystem)$3);}
	| MOTION_ internalSystem numeric numeric
	  {fr->crosshairMotionCmd(Vector($3,$4), (InternalSystem)$2);}
	| internalSystem numeric numeric
	  {fr->crosshairCmd(Vector($2,$3), (InternalSystem)$1);}
	| BEGIN_ MOTION_ coordSystem coord
	  {fr->crosshairBeginCmd(Vector($4), (CoordSystem)$3);}
	| MOTION_ coordSystem coord
	  {fr->crosshairMotionCmd(Vector($3), (CoordSystem)$2);}
	| coordSystem skyFrame coord
	  {fr->crosshairCmd(Vector($3), (CoordSystem)$1, (SkyFrame)$2);}
	| yesno {fr->crosshairCmd($1);}
	| WARP_ numeric numeric {fr->crosshairWarpCmd(Vector($2,$3));}
	;

fitsy	: HAS_ EXT_ STRING {fr->fitsyHasExtCmd($3);}
	;

get	: BG_ COLOR_ {fr->getBgColorCmd();}
	| BIN_ getBin
	| CLIP_ getClip
	| COLORMAP_ {fr->getColormapCmd();}
	| COLORSCALE_ getColorScale
	| CONTOUR_ getContour
	| COORDINATES_ getCoord
	| CROSSHAIR_ getCrosshair
	| CURSOR_ getCursor
	| DATA_ getData
	| DATASEC_ {fr->getDATASECCmd();}
	| FITS_ getFits
	| GRID_ getGrid
	| HISTOGRAM_ STRING STRING {fr->getHistogramCmd($2,$3);}
	| HORIZONTAL_ CUT_ STRING STRING numeric numeric internalSystem
	  {fr->getHorzCutCmd($3,$4,Vector($5,$6),(InternalSystem)$7);}
	| IIS_ getiis
	| INFO_ getInfo
	| MINMAX_ {fr->getMinMaxCmd();}
	| MARKER_ markerLayer markerGet
	| MASK_ getMask
	| NAN_ COLOR_ {fr->getNANColorCmd();}
	| ORIENT_ {fr->getOrientCmd();}
	| PAN_ getCursor
	| PIXEL_ TABLE_ internalSystem numeric numeric INT STRING
	  {fr->getPixelTableCmd(Vector($4,$5), (InternalSystem)$3, $6, $7);}
	| RGB_ getRGB
	| ROTATE_ getRotate
	| SMOOTH_ getSmooth
	| THREED_ getThreed
	| TYPE_ {fr->getTypeCmd();}
	| VALUE_ internalSystem numeric numeric
	  {fr->getValueCmd(Vector($3,$4),(InternalSystem)$2);}
	| VERTICAL_ CUT_ STRING STRING numeric numeric internalSystem
	  {fr->getVertCutCmd($3,$4,Vector($5,$6),(InternalSystem)$7);}
	| WCS_ getWCS
	| ZOOM_ getZoom
	;

getBin	: DEPTH_ {fr->getBinDepthCmd();}
	| FACTOR_ {fr->getBinFactorCmd();}
	| FUNCTION_ {fr->getBinFunctionCmd();}
	| BUFFER_ SIZE_ {fr->getBinBufferSizeCmd();}
	| CURSOR_ {fr->getBinCursorCmd();}
	| FILTER_ {fr->getBinFilterCmd();}
	| COLS_ {fr->getBinColsCmd();}
	| COLS_ MINMAX_ STRING {fr->getBinColsMinMaxCmd($3);}
	| COLS_ DIM_ STRING {fr->getBinColsDimCmd($3);}
	| LIST_ {fr->getBinListCmd();}
	;

getClip	: getClipMode
	| SCOPE_ {fr->getClipScopeCmd();}
	| MODE_ {fr->getClipModeCmd();}
	| MINMAX_ getClipMinMax
	| USER_ LEVEL_ {fr->getClipUserCmd();}
	| ZSCALE_ getClipZScale
	| PRESERVE_ {fr->getClipPreserveCmd();}
	;

getClipMode: /* empty */ {fr->getClipCmd();}
	| numeric {fr->getClipCmd($1);}
	| MINMAX_ {fr->getClipCmd(FrScale::MINMAX);}
	| ZSCALE_ {fr->getClipCmd(FrScale::ZSCALE);}
	| ZMAX_ {fr->getClipCmd(FrScale::ZMAX);}
	| USER_ {fr->getClipCmd(FrScale::USERCLIP);}
	;

getClipMinMax : MODE_ {fr->getClipMinMaxModeCmd();}
	| SAMPLE_ {fr->getClipMinMaxSampleCmd();}
	;

getClipZScale: CONTRAST_ {fr->getClipZScaleContrastCmd();}
	| SAMPLE_ {fr->getClipZScaleSampleCmd();}
	| LINE_ {fr->getClipZScaleLineCmd();}
	;

getColorScale : /* empty */ {fr->getColorScaleCmd();}
	| LEVEL_ getColorScaleLevel
	;

getColorScaleLevel: INT {fr->getColorScaleLevelCmd($1);}
	| INT internalSystem numeric numeric
	  {fr->getColorScaleLevelCmd($1,Vector($3,$4),(InternalSystem)$2);}
	| INT numeric numeric scaleType 
	  {fr->getColorScaleLevelCmd($1,$2,$3,(FrScale::ColorScaleType)$4);}
	;

getContour: coordSystem skyFrame 
	  {fr->getContourCmd((CoordSystem)$1,(SkyFrame)$2);}
	| COLOR_ {fr->getContourColorNameCmd();}
	| DASH_ {fr->getContourDashCmd();}
	| LEVEL_ {fr->getContourLevelCmd();}
	| METHOD_ {fr->getContourMethodCmd();}
	| WIDTH_ {fr->getContourLineWidthCmd();}
	;

getCoord : internalSystem numeric numeric coordSystem skyFrame skyFormat
	  {fr->getCoordCmd(Vector($2,$3), (InternalSystem)$1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	;

getCrosshair: internalSystem {fr->getCrosshairCmd((InternalSystem)$1);}
	| coordSystem skyFrame skyFormat precision
	  {fr->getCrosshairCmd((CoordSystem)$1, (SkyFrame)$2, (SkyFormat)$3,
	    (Precision)$4)}
	| STATUS_ {fr->getCrosshairStatusCmd();}
	;

getCursor : internalSystem {fr->getCursorCmd((InternalSystem)$1);}
	| coordSystem skyFrame skyFormat precision
	  {fr->getCursorCmd((CoordSystem)$1, (SkyFrame)$2, (SkyFormat)$3,
	    (Precision)$4);}
	| PRESERVE_ {fr->getPanPreserveCmd();}
	;

getData	: coordSystem skyFrame coord numeric numeric STRING
	  {fr->getDataValuesCmd(1, Vector($3), 
	    (CoordSystem)$1, (SkyFrame)$2, Vector($4,$5), $6);}
	| INT coordSystem skyFrame coord numeric numeric STRING
	  {fr->getDataValuesCmd($1, Vector($4), 
	    (CoordSystem)$2, (SkyFrame)$3, Vector($5,$6), $7);}
	| internalSystem numeric numeric INT INT
	  {fr->getDataValuesCmd(Vector($2,$3),(InternalSystem)$1,
	    Vector($4,$5));}
	;

getInfo	: STRING {fr->getInfoCmd($1);}
	| CLIP_ {fr->getInfoClipCmd();}
 	| internalSystem numeric numeric skyFrame skyFormat STRING
	  {fr->getInfoCmd(Vector($2,$3), (InternalSystem)$1, 
	    (SkyFrame)$4, (SkyFormat)$5, $6);}
	;

getiis	: POINTER INT INT INT INT {fr->iisGetCmd((char*)$1,$2,$3,$4,$5);}
	| CURSOR_ {fr->iisGetCursorCmd();}
	| WIDTH_ {fr->getWidthCmd();}
	| HEIGHT_ {fr->getHeightCmd();}
	| FILE_ NAME_ getIISFileName
	;

getIISFileName : /* empty */ {fr->iisGetFileNameCmd();}
	| INT {fr->iisGetFileNameCmd($1);}
	| numeric numeric {fr->iisGetFileNameCmd(Vector($1,$2));}
	;

getFits	: CENTER_ coordSystem skyFrame skyFormat precision
	    {fr->getFitsCenterCmd((CoordSystem)$2,(SkyFrame)$3,(SkyFormat)$4,
	      (Precision)$5);}
	| COUNT_ {fr->getFitsCountCmd();}
	| MOSAIC_ COUNT_ {fr->getFitsMosaicCountCmd();}
	| DEPTH_ {fr->getFitsDepthCmd();}
	| BITPIX_ {fr->getBitpixCmd();}
	| EXT_ internalSystem numeric numeric
	  {fr->getFitsExtCmd(Vector($3,$4),(InternalSystem)$2);}
	| FILE_ NAME_ getFitsFileName
	| HEADER_ INT {fr->getFitsHeaderCmd($2);}
	| HEADER_ INT KEYWORD_ STRING {fr->getFitsHeaderKeywordCmd($2,$4);}
	| HEIGHT_ {fr->getHeightCmd();}
	| OBJECT_ NAME_ {fr->getFitsObjectNameCmd();}
	| SIZE_ {fr->getFitsSizeCmd();}
	| SIZE_ coordSystem skyFrame skyFormat precision
	    {fr->getFitsSizeCmd((CoordSystem)$2,(SkyFrame)$3,(SkyFormat)$4,
	      (Precision)$5);}
	| SLICE_ {fr->getFitsSliceCmd();}
	| WIDTH_ {fr->getWidthCmd();}
	;

getFitsFileName: fileNameType 
	  {fr->getFitsFileNameCmd((Base::FileNameType)$1);}
	| fileNameType internalSystem numeric numeric
	  {fr->getFitsFileNameCmd(Vector($3,$4), (InternalSystem)$2,
	    (Base::FileNameType)$1);}
	| fileNameType INT 
	  {fr->getFitsFileNameCmd($2, (Base::FileNameType)$1);}
	;

getGrid : /* empty */ {fr->getGridCmd();}
	| OPTION_ {fr->getGridOptionCmd();}
	;

getMask	: COLOR_ {fr->getMaskColorCmd();}
	| MARK_ {fr->getMaskMarkCmd();}
	| TRANSPARENCY_{fr->getMaskTransparencyCmd();}
	;

getRGB	: CHANNEL_ {fr->getRGBChannelCmd();}
	| SYSTEM_ {fr->getRGBSystemCmd();}
	| VIEW_ {fr->getRGBViewCmd();}
	;

getRotate : precision {fr->getRotateCmd((Precision)$1);}
	;

getSmooth : FUNCTION_  {fr->getSmoothFunctionCmd();}
	| RADIUS_ {fr->getSmoothRadiusCmd();}
	;

getThreed : VIEW_ {fr->get3dViewCmd();}
	| METHOD_ {fr->get3dRenderMethodCmd();}
	| THRESHOLD_ {fr->get3dThresholdCmd();}
	;

getWCS	: ALIGN_ {fr->getWCSAlignCmd();}
	| NAME_ wcsSystem {fr->getWCSNameCmd((CoordSystem)$2);}
	;

getZoom	: precision {fr->getZoomCmd((Precision)$1);}
	;

grid	: CREATE_ coordSystem skyFrame skyFormat gridType STRING
	   {fr->gridCmd((CoordSystem)$2, (SkyFrame)$3, (SkyFormat)$4,
	     (Grid::GridType)$5, $6);}
	| DELETE_ {fr->gridDeleteCmd();}
	;

gridType : ANALYSIS_ {$$=Grid::ANALYSIS;}
	| PUBLICATION_ {$$=Grid::PUBLICATION;}
	;

has	: AMPLIFIER_ {fr->hasAmplifierCmd();}
	| CONTOUR_ {fr->hasContourCmd();}
	| DATAMIN_ {fr->hasDATAMINCmd();}
	| DATASEC_ {fr->hasDATASECCmd();}
	| DETECTOR_ {fr->hasDetectorCmd();}
	| FITS_ hasFits
	| GRID_ {fr->hasGridCmd();}
	| IIS_ {fr->hasIISCmd();}
	| IRAFMIN_ {fr->hasIRAFMINCmd();}
	| MARKER_ hasMarker
	| PHYSICAL_ {fr->hasPhysicalCmd();}
	| SMOOTH_ {fr->hasSmoothCmd();}
	| SYSTEM_ coordSystem {fr->hasSystemCmd((CoordSystem)$2);}
	| WCS_ hasWCS
	;

hasFits	: /* empty */ {fr->hasFitsCmd();}
	| BIN_ {fr->hasFitsBinCmd();}
	| CUBE_ {fr->hasFitsCubeCmd();}
	| MOSAIC_ {fr->hasFitsMosaicCmd();}
	;

hasMarker : HIGHLITE_ {fr->hasMarkerHighlitedCmd();}
	| SELECT_ {fr->hasMarkerSelectedCmd();}
	| PASTE_ {fr->hasMarkerPasteCmd();}
	| UNDO_ {fr->hasMarkerUndoCmd();}
	;

hasWCS	: coordSystem {fr->hasWCSCmd((CoordSystem)$1);}
	| EQUATORIAL_ coordSystem {fr->hasWCSEquCmd((CoordSystem)$2);}
	| LINEAR_ coordSystem {fr->hasWCSLinearCmd((CoordSystem)$2);}
	;

iis	: NEW_ INT INT {fr->iisCmd($2,$3);}
	| ERASE_ {fr->iisEraseCmd();}
	| MESSAGE_ STRING {fr->iisMessageCmd($2);}
	| CURSOR_ iiscursor
	| SET_ FILE_ NAME_ iisSetFileName
	| SET_ POINTER INT INT INT INT
	  {fr->iisSetCmd((const char*)$2,$3,$4,$5,$6);}
	| UPDATE_ {fr->iisUpdateCmd();}
	| WCS_ numeric numeric numeric numeric 
	  numeric numeric numeric numeric INT
	    {fr->iisWCSCmd(Matrix($2,$3,$4,$5,$6,$7),Vector($8,$9),$10);}
	;

iisSetFileName : STRING {fr->iisSetFileNameCmd($1);}
	| STRING INT {fr->iisSetFileNameCmd($1,$2);}
	;

iiscursor: INT INT CANVAS_
	  {fr->iisSetCursorCmd(Vector($1,$2),CANVAS);}
	| INT INT coordSystem
	  {fr->iisSetCursorCmd(Vector($1,$2),(CoordSystem)$3);}
	| MODE_ yesno {fr->iisCursorModeCmd($2);}
	;

load	: ARRAY_ loadArr
	| FITS_ loadFits
	| INCR_ loadIncr
	;

loadArr : STRING ALLOC_ STRING layerType 
	  {fr->loadArrAllocCmd($3, $1, (Base::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType 
	  {fr->loadArrAllocGZCmd($3, $1, (Base::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType 
	  {fr->loadArrChannelCmd($3, $1, (Base::LayerType)$4);}
	| STRING MMAP_ layerType 
	  {fr->loadArrMMapCmd($1, (Base::LayerType)$3);}
	| STRING MMAPINCR_ layerType 
	  {fr->loadArrMMapIncrCmd($1, (Base::LayerType)$3);}
	| STRING SHARED_ shmType INT layerType
	  {fr->loadArrShareCmd((Base::ShmType)$3, $4, $1,
	    (Base::LayerType)$5);}
	| STRING SOCKET_ INT layerType 
	  {fr->loadArrSocketCmd($3, $1, (Base::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType 
	  {fr->loadArrSocketGZCmd($3, $1, (Base::LayerType)$4);}
	| STRING VAR_ STRING layerType 
	  {fr->loadArrVarCmd($3, $1, (Base::LayerType)$4);}
	| RGB_ CUBE_ loadArrRGBCube
	;

loadArrRGBCube : STRING ALLOC_ STRING {fr->loadArrRGBCubeAllocCmd($3, $1);}
	| STRING ALLOCGZ_ STRING {fr->loadArrRGBCubeAllocGZCmd($3, $1);}
	| STRING CHANNEL_ STRING {fr->loadArrRGBCubeChannelCmd($3, $1);}
	| STRING MMAP_ {fr->loadArrRGBCubeMMapCmd($1);}
	| STRING MMAPINCR_  {fr->loadArrRGBCubeMMapIncrCmd($1);}
	| STRING SHARED_ shmType INT
	  {fr->loadArrRGBCubeShareCmd((Base::ShmType)$3, $4, $1);}
	| STRING SOCKET_ INT {fr->loadArrRGBCubeSocketCmd($3, $1);}
	| STRING SOCKETGZ_ INT {fr->loadArrRGBCubeSocketGZCmd($3, $1);}
	| STRING VAR_ STRING {fr->loadArrRGBCubeVarCmd($3, $1);}
	;

loadFits: STRING ALLOC_ STRING layerType 
	  {fr->loadFitsAllocCmd($3, $1, (Base::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType 
	  {fr->loadFitsAllocGZCmd($3, $1, (Base::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType 
	  {fr->loadFitsChannelCmd($3, $1, (Base::LayerType)$4);}
	| STRING MMAP_ incrLoad layerType
	  {fr->loadFitsMMapCmd($1,(Base::LoadMethod)$3,
	    (Base::LayerType)$4);}
	| STRING STRING SMMAP_ incrLoad layerType
	  {fr->loadFitsSMMapCmd($1, $2,(Base::LoadMethod)$4,	
	    (Base::LayerType)$5);}
	| STRING MMAPINCR_ incrLoad layerType
	  {fr->loadFitsMMapIncrCmd($1,(Base::LoadMethod)$3,
	    (Base::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {fr->loadFitsShareCmd((Base::ShmType)$3, $4, $1, 
	    (Base::LoadMethod)$5,(Base::LayerType)$6);}
	| STRING SSHARED_ shmType INT INT incrLoad layerType
	  {fr->loadFitsSShareCmd((Base::ShmType)$3, $4, $5, $1, 
	    (Base::LoadMethod)$6,(Base::LayerType)$7);}
	| STRING SOCKET_ INT  layerType
	  {fr->loadFitsSocketCmd($3, $1, (Base::LayerType)$4);}
	| STRING SOCKETGZ_ INT  layerType
	  {fr->loadFitsSocketGZCmd($3, $1, (Base::LayerType)$4);}
	| STRING VAR_ STRING incrLoad layerType
	  {fr->loadFitsVarCmd($3, $1, (Base::LoadMethod)$4,
	    (Base::LayerType)$5);}
	| DATA_ CUBE_ loadFitsDataCube
	| MOSAIC_ loadFitsMosaic
	| RGB_ loadFitsRGB
	;

loadFitsDataCube: STRING ALLOC_ STRING layerType
	  {fr->loadDataCubeAllocCmd($3, $1, (Base::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType 
	  {fr->loadDataCubeAllocGZCmd($3, $1, (Base::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType
	  {fr->loadDataCubeChannelCmd($3, $1, (Base::LayerType)$4);}
	| STRING MMAP_ incrLoad layerType
	  {fr->loadDataCubeMMapCmd($1, 
	    (Base::LoadMethod)$3, (Base::LayerType)$4);}
	| STRING MMAPINCR_ incrLoad layerType
	  {fr->loadDataCubeMMapIncrCmd($1, 
	    (Base::LoadMethod)$3, (Base::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {fr->loadDataCubeShareCmd((Base::ShmType)$3, $4, $1, 
	    (Base::LoadMethod)$5, (Base::LayerType)$6);}
	| STRING SOCKET_ INT layerType
	  {fr->loadDataCubeSocketCmd($3, $1, (Base::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType
	  {fr->loadDataCubeSocketGZCmd($3, $1, (Base::LayerType)$4);}
	| STRING VAR_ STRING incrLoad layerType
	  {fr->loadDataCubeVarCmd($3, $1, 
	    (Base::LoadMethod)$4, (Base::LayerType)$5);}
	;

loadFitsMosaic : IMAGE_ IRAF_ loadFitsMosaicImageIRAF
	| IRAF_ loadFitsMosaicIRAF
	| IMAGE_ loadFitsMosaicImageWCS
	| loadFitsMosaicWCS
	| IMAGE_ WFPC2_ loadFitsMosaicImageWFPC2
	;

loadFitsMosaicImageIRAF : STRING ALLOC_ STRING layerType
	  {fr->loadMosaicImageIRAFAllocCmd($3, $1, (Base::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType
	  {fr->loadMosaicImageIRAFAllocGZCmd($3, $1,(Base::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType
	  {fr->loadMosaicImageIRAFChannelCmd($3, $1,(Base::LayerType)$4);}
	| STRING MMAP_ incrLoad layerType
	  {fr->loadMosaicImageIRAFMMapCmd($1,(Base::LoadMethod)$3,
	    (Base::LayerType)$4);}
	| STRING MMAPINCR_ incrLoad layerType
	  {fr->loadMosaicImageIRAFMMapIncrCmd($1,(Base::LoadMethod)$3,
	    (Base::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {fr->loadMosaicImageIRAFShareCmd((Base::ShmType)$3, $4,
	    $1, (Base::LoadMethod)$5, (Base::LayerType)$6);}
	| STRING SOCKET_ INT layerType
	  {fr->loadMosaicImageIRAFSocketCmd($3, $1, (Base::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType
	  {fr->loadMosaicImageIRAFSocketGZCmd($3, $1, 
	    (Base::LayerType)$4);}
	| STRING VAR_ STRING incrLoad layerType
	  {fr->loadMosaicImageIRAFVarCmd($3, $1, (Base::LoadMethod)$4, 
	    (Base::LayerType)$5);}
	;

loadFitsMosaicIRAF : STRING ALLOC_ STRING layerType
	  {fr->loadMosaicIRAFAllocCmd($3, $1, (Base::LayerType)$4);}
	| STRING ALLOCGZ_ STRING layerType
	  {fr->loadMosaicIRAFAllocGZCmd($3, $1, (Base::LayerType)$4);}
	| STRING CHANNEL_ STRING layerType
	  {fr->loadMosaicIRAFChannelCmd($3, $1, (Base::LayerType)$4);}
	| STRING MMAP_ incrLoad layerType
	  {fr->loadMosaicIRAFMMapCmd($1,(Base::LoadMethod)$3, 
	    (Base::LayerType)$4);}
	| STRING STRING SMMAP_ incrLoad layerType
	  {fr->loadMosaicIRAFSMMapCmd($1, $2,(Base::LoadMethod)$4, 
	    (Base::LayerType)$5);}
	| STRING MMAPINCR_ incrLoad layerType
	  {fr->loadMosaicIRAFMMapIncrCmd($1,(Base::LoadMethod)$3, 
	    (Base::LayerType)$4);}
	| STRING SHARED_ shmType INT incrLoad layerType
	  {fr->loadMosaicIRAFShareCmd((Base::ShmType)$3, $4,
	    $1, (Base::LoadMethod)$5, (Base::LayerType)$6);}
	| STRING SSHARED_ shmType INT INT incrLoad layerType
	  {fr->loadMosaicIRAFSShareCmd((Base::ShmType)$3, $4, $5,
	    $1, (Base::LoadMethod)$6, (Base::LayerType)$7);}
	| STRING SOCKET_ INT layerType
	  {fr->loadMosaicIRAFSocketCmd($3, $1, (Base::LayerType)$4);}
	| STRING SOCKETGZ_ INT layerType
	  {fr->loadMosaicIRAFSocketGZCmd($3, $1, (Base::LayerType)$4);}
	| STRING VAR_ STRING incrLoad layerType
	  {fr->loadMosaicIRAFVarCmd($3, $1, (Base::LoadMethod)$4, 
	    (Base::LayerType)$5);}
	;

loadFitsMosaicImageWCS : wcsSystem STRING ALLOC_ STRING layerType
	  {fr->loadMosaicImageWCSAllocCmd((CoordSystem)$1, $4, $2, 
	    (Base::LayerType)$5);}
	| wcsSystem STRING ALLOCGZ_ STRING layerType
	  {fr->loadMosaicImageWCSAllocGZCmd((CoordSystem)$1, $4, $2,
	    (Base::LayerType)$5);}
	| wcsSystem STRING CHANNEL_ STRING layerType
	  {fr->loadMosaicImageWCSChannelCmd((CoordSystem)$1, $4, $2,
  	    (Base::LayerType)$5);}
	| wcsSystem STRING MMAP_ incrLoad layerType
	  {fr->loadMosaicImageWCSMMapCmd((CoordSystem)$1, $2,
	    (Base::LoadMethod)$4, (Base::LayerType)$5);}
	| wcsSystem STRING MMAPINCR_ incrLoad layerType
	  {fr->loadMosaicImageWCSMMapIncrCmd((CoordSystem)$1, $2,
	    (Base::LoadMethod)$4, (Base::LayerType)$5);}
	| wcsSystem STRING SHARED_ shmType INT incrLoad layerType
	  {fr->loadMosaicImageWCSShareCmd((CoordSystem)$1, 
	    (Base::ShmType)$4, $5, $2, (Base::LoadMethod)$6,
	    (Base::LayerType)$7);}
	| wcsSystem STRING SOCKET_ INT layerType
	  {fr->loadMosaicImageWCSSocketCmd((CoordSystem)$1, $4, $2,	
	    (Base::LayerType)$5);}
	| wcsSystem STRING SOCKETGZ_ INT layerType
	  {fr->loadMosaicImageWCSSocketGZCmd((CoordSystem)$1, $4, $2,
	    (Base::LayerType)$5);}
	| wcsSystem STRING VAR_ STRING incrLoad layerType
	  {fr->loadMosaicImageWCSVarCmd((CoordSystem)$1, $4, $2, 
	    (Base::LoadMethod)$5, (Base::LayerType)$6);}
	;

loadFitsMosaicWCS : wcsSystem STRING ALLOC_ STRING layerType
	  {fr->loadMosaicWCSAllocCmd((CoordSystem)$1, $4, $2, 
	    (Base::LayerType)$5);}
	| wcsSystem STRING ALLOCGZ_ STRING layerType
	  {fr->loadMosaicWCSAllocGZCmd((CoordSystem)$1, $4, $2,
	    (Base::LayerType)$5);}
	| wcsSystem STRING CHANNEL_ STRING  layerType
	  {fr->loadMosaicWCSChannelCmd((CoordSystem)$1, $4, $2,
	    (Base::LayerType)$5);}
	| wcsSystem STRING MMAP_ incrLoad  layerType
	  {fr->loadMosaicWCSMMapCmd((CoordSystem)$1, $2,
	    (Base::LoadMethod)$4, (Base::LayerType)$5);}
	| wcsSystem STRING STRING SMMAP_ incrLoad  layerType
	  {fr->loadMosaicWCSSMMapCmd((CoordSystem)$1, $2, $3,
	    (Base::LoadMethod)$5, (Base::LayerType)$6);}
	| wcsSystem STRING MMAPINCR_ incrLoad  layerType
	  {fr->loadMosaicWCSMMapIncrCmd((CoordSystem)$1, $2,
	    (Base::LoadMethod)$4, (Base::LayerType)$5);}
	| wcsSystem STRING SHARED_ shmType INT incrLoad layerType
	  {fr->loadMosaicWCSShareCmd((CoordSystem)$1, 
	    (Base::ShmType)$4, $5, $2, (Base::LoadMethod)$6,
	    (Base::LayerType)$7);}
	| wcsSystem STRING SSHARED_ shmType INT INT incrLoad layerType
	  {fr->loadMosaicWCSSShareCmd((CoordSystem)$1, 
	    (Base::ShmType)$4, $5, $6, $2, (Base::LoadMethod)$7,
	    (Base::LayerType)$8);}
	| wcsSystem STRING SOCKET_ INT  layerType
	  {fr->loadMosaicWCSSocketCmd((CoordSystem)$1, $4, $2,
	    (Base::LayerType)$5);}
	| wcsSystem STRING SOCKETGZ_ INT  layerType
	  {fr->loadMosaicWCSSocketGZCmd((CoordSystem)$1, $4, $2,
	    (Base::LayerType)$5);}
	| wcsSystem STRING VAR_ STRING incrLoad layerType
	  {fr->loadMosaicWCSVarCmd((CoordSystem)$1, $4, $2, 
	    (Base::LoadMethod)$5, (Base::LayerType)$6);}
	;

loadFitsMosaicImageWFPC2 : STRING ALLOC_ STRING
	  {fr->loadMosaicImageWFPC2AllocCmd($3, $1);}
	| STRING ALLOCGZ_ STRING
	  {fr->loadMosaicImageWFPC2AllocGZCmd($3, $1);}
	| STRING CHANNEL_ STRING
	  {fr->loadMosaicImageWFPC2ChannelCmd($3, $1);}
	| STRING MMAP_ incrLoad
	  {fr->loadMosaicImageWFPC2MMapCmd($1,(Base::LoadMethod)$3);}
	| STRING MMAPINCR_ incrLoad
	  {fr->loadMosaicImageWFPC2MMapIncrCmd($1,(Base::LoadMethod)$3);}
	| STRING SHARED_ shmType INT incrLoad
	  {fr->loadMosaicImageWFPC2ShareCmd((Base::ShmType)$3, $4,
	    $1, (Base::LoadMethod)$5);}
	| STRING SOCKET_ INT
	  {fr->loadMosaicImageWFPC2SocketCmd($3, $1);}
	| STRING SOCKETGZ_ INT
	  {fr->loadMosaicImageWFPC2SocketGZCmd($3, $1);}
	| STRING VAR_ STRING incrLoad
	  {fr->loadMosaicImageWFPC2VarCmd($3, $1, (Base::LoadMethod)$4);}
	;
loadFitsRGB : CUBE_ loadFitsRGBCube
	| IMAGE_ loadFitsRGBImage
	;

loadFitsRGBCube: STRING ALLOC_ STRING {fr->loadRGBCubeAllocCmd($3, $1);}
	| STRING ALLOCGZ_ STRING {fr->loadRGBCubeAllocGZCmd($3, $1);}
	| STRING CHANNEL_ STRING {fr->loadRGBCubeChannelCmd($3, $1);}
	| STRING MMAP_ incrLoad
	  {fr->loadRGBCubeMMapCmd($1, (Base::LoadMethod)$3);}
	| STRING STRING SMMAP_ incrLoad
	  {fr->loadRGBCubeSMMapCmd($1, $2, (Base::LoadMethod)$4);}
	| STRING MMAPINCR_ incrLoad
	  {fr->loadRGBCubeMMapIncrCmd($1, (Base::LoadMethod)$3);}
	| STRING SHARED_ shmType INT incrLoad
	  {fr->loadRGBCubeShareCmd((Base::ShmType)$3, $4, $1, 
	    (Base::LoadMethod)$5);}
	| STRING SSHARED_ shmType INT INT incrLoad
	  {fr->loadRGBCubeSShareCmd((Base::ShmType)$3, $4, $5, $1, 
	    (Base::LoadMethod)$6);}
	| STRING SOCKET_ INT {fr->loadRGBCubeSocketCmd($3, $1);}
	| STRING SOCKETGZ_ INT {fr->loadRGBCubeSocketGZCmd($3, $1);}
	| STRING VAR_ STRING incrLoad
	  {fr->loadRGBCubeVarCmd($3, $1, (Base::LoadMethod)$4);}
	;

loadFitsRGBImage: STRING ALLOC_ STRING {fr->loadRGBImageAllocCmd($3, $1);}
	| STRING ALLOCGZ_ STRING {fr->loadRGBImageAllocGZCmd($3, $1);}
	| STRING CHANNEL_ STRING {fr->loadRGBImageChannelCmd($3, $1);}
	| STRING MMAP_ incrLoad
	  {fr->loadRGBImageMMapCmd($1, (Base::LoadMethod)$3);}
	| STRING MMAPINCR_ incrLoad
	  {fr->loadRGBImageMMapIncrCmd($1, (Base::LoadMethod)$3);}
	| STRING SHARED_ shmType INT incrLoad
	  {fr->loadRGBImageShareCmd((Base::ShmType)$3, $4, $1, 
	    (Base::LoadMethod)$5);}
	| STRING SOCKET_ INT {fr->loadRGBImageSocketCmd($3, $1);}
	| STRING SOCKETGZ_ INT {fr->loadRGBImageSocketGZCmd($3, $1);}
	| STRING VAR_ STRING incrLoad
	  {fr->loadRGBImageVarCmd($3, $1, (Base::LoadMethod)$4);}
	;

loadIncr: DATA_ INT INT INT INT INT {fr->loadIncrDataCmd($2,$3,$4,$5,$6);}
	| MINMAX_ INT INT INT INT INT {fr->loadIncrMinMaxCmd($2,$3,$4,$5,$6);}
	| END_ {fr->loadIncrEndCmd();}
	;

macosx	: PRINT_ {
#ifdef _MACOSX
	  fr->macosxPrintCmd();
#endif
	}
	;

magnifier: yesno {fr->magnifierCmd($1);}
	| GRAPHICS_ yesno {fr->magnifierGraphicsCmd($2);}
	| CURSOR_ yesno {fr->magnifierCursorCmd($2);}
	| STRING INT INT {fr->magnifierCmd($1, $2, $3);}
	| UPDATE_ {fr->updateMagnifierCmd();}
	| UPDATE_ numeric numeric {fr->updateMagnifierCmd(Vector($2, $3));}
	| ZOOM_ numeric {fr->magnifierZoomCmd($2);}
	;

marker	: CENTROID_ markerCentroid
	| COLOR_ STRING {fr->markerColorCmd($2);}
	| COPY_ {fr->markerCopyCmd();}
	| COMMAND_ markerFormat STRING
	  {fr->markerCommandCmd((Base::MarkerFormat)$2,$3);}
	| COMMAND_ markerFormat VAR_ STRING
	  {fr->markerCommandVarCmd((Base::MarkerFormat)$2,$4);}
	| COMPOSITE_ DELETE_ {fr->markerCompositeDeleteCmd();}
	| CREATE_ markerCreate
	| CUT_ {fr->markerCutCmd();}
	| DELETE_ {fr->markerDeleteCmd();}
	| DELETE_ ALL_ {fr->markerDeleteAllCmd();}
	| EDIT_ markerEdit
	| FONT_ STRING {fr->markerFontCmd($2);}

	| HIGHLITE_ ALL_ {fr->markerHighliteAllCmd();}
	| HIGHLITE_ ONLY_ numeric numeric 
	  {fr->markerHighliteOnlyCmd(Vector($3,$4));}
	| HIGHLITE_ TOGGLE_ numeric numeric
	  {fr->markerHighliteToggleCmd(Vector($3,$4));}

	| INT ANGLE_ angle {fr->markerAngleCmd($1,$3);}
	| INT ANGLE_ angle internalSystem {fr->markerAngleCmd($1,$3);}
	| INT ANGLE_ angle coordSystem skyFrame
	  {fr->markerAngleCmd($1,$3,(CoordSystem)$4, (SkyFrame)$5);}

	| INT ANNULUS_ RADIUS_ numeric numeric INT internalSystem
	  {fr->markerAnnulusRadiusCmd($1, $4, $5, $6, (InternalSystem)$7);}
	| INT ANNULUS_ RADIUS_ numeric numeric INT coordSystem skyFormat
	  {fr->markerAnnulusRadiusCmd($1, $4, $5, $6,
	    (CoordSystem)$7, (SkyFormat)$8);}
	| INT ANNULUS_ RADIUS_ STRING coordSystem skyFormat
	  {fr->markerAnnulusRadiusCmd($1, $4,(CoordSystem)$5,(SkyFormat)$6);}

	| INT BOXANNULUS_ RADIUS_ numeric numeric numeric INT internalSystem
	  {fr->markerBoxAnnulusRadiusCmd($1, Vector($4, $5), 
	    Vector($6, $6*$5/$4), $7, (InternalSystem)$8);}
	| INT BOXANNULUS_ RADIUS_ numeric numeric numeric INT 
	    coordSystem skyFormat
	  {fr->markerBoxAnnulusRadiusCmd($1, Vector($4, $5), 
	    Vector($6, $6*$5/$4), $7, (CoordSystem)$8, (SkyFormat)$9);}
	| INT BOXANNULUS_ RADIUS_ STRING coordSystem skyFormat
	  {fr->markerBoxAnnulusRadiusCmd($1,$4,(CoordSystem)$5,(SkyFormat)$6);}

	| INT BOX_ RADIUS_ numeric numeric internalSystem
	  {fr->markerBoxRadiusCmd($1, Vector($4,$5), (InternalSystem)$6);}
	| INT BOX_ RADIUS_ numeric numeric coordSystem skyFormat
	  {fr->markerBoxRadiusCmd($1, Vector($4,$5), 
	    (CoordSystem)$6, (SkyFormat)$7);}

	| INT BPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	  {fr->markerBpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10);}
	| INT BPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	    internalSystem
	  {fr->markerBpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10);}
	| INT BPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	    coordSystem skyFrame
	  {fr->markerBpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10,
	    (CoordSystem)$11, (SkyFrame)$12);}
	| INT BPANDA_ EDIT_ STRING STRING coordSystem skyFrame 
	    coordSystem skyFormat
	  {fr->markerBpandaEditCmd($1, $4, $5, 
	    (CoordSystem)$6, (SkyFrame)$7, (CoordSystem)$8, (SkyFormat)$9);}

	| INT CALLBACK_ markerCallBack STRING STRING
	  {fr->markerCallBackCmd($1,(CallBack::Type)$3,$4,$5);}

	| INT CIRCLE_ RADIUS_ numeric internalSystem
	  {fr->markerCircleRadiusCmd($1, $4, (InternalSystem)$5);}
	| INT CIRCLE_ RADIUS_ numeric coordSystem skyFormat
	  {fr->markerCircleRadiusCmd($1, $4, (CoordSystem)$5, (SkyFormat)$6);}

	| INT COLOR_ STRING {fr->markerColorCmd($1,$3);}

	| INT COMPASS_ ARROW_ yesno yesno 
	  {fr->markerCompassArrowCmd($1,$4,$5);}
	| INT COMPASS_ LABEL_ STRING STRING
	  {fr->markerCompassLabelCmd($1,$4,$5);}
	| INT COMPASS_ RADIUS_ numeric internalSystem
	  {fr->markerCompassRadiusCmd($1,$4,(InternalSystem)$5);}
	| INT COMPASS_ RADIUS_ numeric coordSystem skyFormat
	  {fr->markerCompassRadiusCmd($1,$4,(CoordSystem)$5,(SkyFormat)$6);}
	| INT COMPASS_ SYSTEM_ coordSystem skyFrame
	  {fr->markerCompassSystemCmd($1, (CoordSystem)$4, (SkyFrame)$5);}

	| INT COMPOSITE_ GLOBAL_ yesno {fr->markerCompositeCmd($1,$4);}

	| INT CPANDA_ EDIT_ angle angle INT numeric numeric INT 
	  {fr->markerCpandaEditCmd($1, $4, $5, $6, $7, $8, $9);}
	| INT CPANDA_ EDIT_ angle angle INT numeric numeric INT internalSystem
	  {fr->markerCpandaEditCmd($1, $4, $5, $6, $7, $8, $9);}
	| INT CPANDA_ EDIT_ angle angle INT numeric numeric INT 
	    coordSystem skyFrame
	  {fr->markerCpandaEditCmd($1, $4, $5, $6, $7, $8, $9,
	    (CoordSystem)$10, (SkyFrame)$11);}
	| INT CPANDA_ EDIT_ STRING STRING coordSystem skyFrame 
	    coordSystem skyFormat
	  {fr->markerCpandaEditCmd($1, $4, $5, (CoordSystem)$6, (SkyFrame)$7,
	    (CoordSystem)$8, (SkyFormat)$9);}

	| INT CREATE_ ANNULUS_ RADIUS_ numeric numeric
	  {fr->markerAnnulusCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ BOXANNULUS_ RADIUS_ numeric numeric
	  {fr->markerBoxAnnulusCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ BPANDA_ ANGLE_ numeric numeric
	  {fr->markerBpandaCreateAnglesCmd($1,Vector($5,$6));}
	| INT CREATE_ BPANDA_ RADIUS_ numeric numeric
	  {fr->markerBpandaCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ ELLIPSEANNULUS_ RADIUS_ numeric numeric
	  {fr->markerEllipseAnnulusCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ EPANDA_ ANGLE_ numeric numeric
	  {fr->markerEpandaCreateAnglesCmd($1,Vector($5,$6));}
	| INT CREATE_ EPANDA_ RADIUS_ numeric numeric
	  {fr->markerEpandaCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ CPANDA_ ANGLE_ numeric numeric
	  {fr->markerCpandaCreateAnglesCmd($1,Vector($5,$6));}
	| INT CREATE_ CPANDA_ RADIUS_ numeric numeric
	  {fr->markerCpandaCreateRadiusCmd($1,Vector($5,$6));}
	| INT CREATE_ POLYGON_ VERTEX_ INT numeric numeric
	  {fr->markerPolygonCreateVertexCmd($1,$5,Vector($6,$7));}

	| INT DELETE_ {fr->markerDeleteCmd($1);}
	| INT DELETE_ ANNULUS_ INT 
	  {fr->markerAnnulusDeleteRadiusCmd($1,$4);}
	| INT DELETE_ BOXANNULUS_ INT 
	  {fr->markerBoxAnnulusDeleteRadiusCmd($1,$4);}
	| INT DELETE_ BPANDA_ INT {fr->markerBpandaDeleteCmd($1,$4);}
	| INT DELETE_ ELLIPSEANNULUS_ INT 
	  {fr->markerEllipseAnnulusDeleteRadiusCmd($1,$4);}
	| INT DELETE_ CALLBACK_ markerCallBack STRING
	  {fr->markerDeleteCallBackCmd($1,(CallBack::Type)$4,$5);}
	| INT DELETE_ EPANDA_ INT {fr->markerEpandaDeleteCmd($1,$4);}
	| INT DELETE_ CPANDA_ INT {fr->markerCpandaDeleteCmd($1,$4);}
	| INT DELETE_ POLYGON_ VERTEX_ INT 
	  {fr->markerPolygonDeleteVertexCmd($1,$5);}
	| INT DELETE_ TAG_ {fr->markerDeleteTagCmd($1);}
	| INT DELETE_ TAG_ STRING {fr->markerDeleteTagCmd($1,$4);}
	| INT DELETE_ TAG_ INT {fr->markerDeleteTagCmd($1,$4);}

	| INT EDIT_ BEGIN_ INT {fr->markerEditBeginCmd($1,$4);}

	| INT ELLIPSE_ RADIUS_ numeric numeric internalSystem
	  {fr->markerEllipseRadiusCmd($1, Vector($4, $5), (InternalSystem)$6);}
	| INT ELLIPSE_ RADIUS_ numeric numeric coordSystem skyFormat
	  {fr->markerEllipseRadiusCmd($1, Vector($4, $5),
	    (CoordSystem)$6, (SkyFormat)$7);}

	| INT ELLIPSEANNULUS_ RADIUS_ numeric numeric numeric INT
	    internalSystem
	  {fr->markerEllipseAnnulusRadiusCmd($1, Vector($4,$5), 
	    Vector($6,$6*$5/$4), $7, (InternalSystem)$8);}
	| INT ELLIPSEANNULUS_ RADIUS_ numeric numeric numeric INT
	    coordSystem skyFormat
	  {fr->markerEllipseAnnulusRadiusCmd($1, Vector($4,$5), 
	    Vector($6,$6*$5/$4), $7, (CoordSystem)$8, (SkyFormat)$9);}
	| INT ELLIPSEANNULUS_ RADIUS_ STRING coordSystem skyFormat
	  {fr->markerEllipseAnnulusRadiusCmd($1, $4,
	    (CoordSystem)$5, (SkyFormat)$6);}

	| INT EPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	  {fr->markerEpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10);}
	| INT EPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	    internalSystem
	  {fr->markerEpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10);}
	| INT EPANDA_ EDIT_ angle angle INT numeric numeric numeric INT 
	    coordSystem skyFrame
	  {fr->markerEpandaEditCmd($1, $4, $5, $6,
	    Vector($7,$8), Vector($9,$9*$8/$7), $10,
	    (CoordSystem)$11, (SkyFrame)$12);}
	| INT EPANDA_ EDIT_ STRING STRING coordSystem skyFrame 
	    coordSystem skyFormat
	  {fr->markerEpandaEditCmd($1, $4, $5, 
	    (CoordSystem)$6, (SkyFrame)$7,
	    (CoordSystem)$8, (SkyFormat)$9);}

	| INT FONT_ STRING {fr->markerFontCmd($1,$3);}
	| INT HIGHLITE_ {fr->markerHighliteCmd($1);}
	| INT HIGHLITE_ ONLY_{fr->markerHighliteOnlyCmd($1);}

	| INT LINE_ ARROW_ yesno yesno {fr->markerLineArrowCmd($1,$4,$5);}
	| INT LINE_ POINT_ internalSystem coord coord
	  {fr->markerLineCmd($1, Vector($5), Vector($6), (InternalSystem)$4);}
	| INT LINE_ POINT_ coordSystem skyFrame coord coord
	  {fr->markerLineCmd($1, Vector($6), Vector($7), 
	    (CoordSystem)$4, (SkyFrame)$5);}

	| INT MOVE_ numeric numeric 
	  {fr->markerMoveCmd($1, Vector($3,$4));}
	| INT MOVE_ FRONT_ {fr->markerFrontCmd($1);}
	| INT MOVE_ BACK_ {fr->markerBackCmd($1);}
	| INT MOVE_ TO_ internalSystem numeric numeric
	  {fr->markerMoveToCmd($1, Vector($5,$6), (InternalSystem)$4);}
	| INT MOVE_ TO_ coordSystem skyFrame coord 
	  {fr->markerMoveToCmd($1,Vector($6),(CoordSystem)$4,(SkyFrame)$5);}

	| INT POLYGON_ RESET_ numeric numeric internalSystem
	  {fr->markerPolygonResetCmd($1, Vector($4,$5),(InternalSystem)$6);}
	| INT POLYGON_ RESET_ numeric numeric coordSystem skyFormat
	  {fr->markerPolygonResetCmd($1, Vector($4,$5),
	    (CoordSystem)$6, (SkyFormat)$7);}

	| INT POINT_ SIZE_ INT {fr->markerPointSizeCmd($1,$4);}

	| INT PROJECTION_ internalSystem coord coord numeric INT 
	  {fr->markerProjectionCmd($1, Vector($4), Vector($5),
	    (InternalSystem)$3, $6, $7);}
	| INT PROJECTION_ coordSystem skyFrame coord coord numeric INT 
	    coordSystem skyFormat
	  {fr->markerProjectionCmd($1, Vector($5), Vector($6),
	    (CoordSystem)$3, (SkyFrame)$4, $7, $8,
	    (CoordSystem)$9, (SkyFormat)$10);}
	| INT PROJECTION_ SYSTEM_ coordSystem skyFrame
	  {fr->markerProjectionSystemCmd($1,(CoordSystem)$4,(SkyFrame)$5);}

	| INT PROPERTY_ markerProperty yesno 
	  {fr->markerPropertyCmd($1,$3,$4);}
	| INT ROTATE_ BEGIN_ {fr->markerRotateBeginCmd($1);}

	| INT RULER_ POINT_ internalSystem coord coord
	  {fr->markerRulerPointCmd($1, Vector($5), Vector($6),
	    (InternalSystem)$4);}
	| INT RULER_ POINT_ coordSystem skyFrame coord coord
	  {fr->markerRulerPointCmd($1, Vector($6), Vector($7),
	    (CoordSystem)$4, (SkyFrame)$5);}
	| INT RULER_ SYSTEM_ coordSystem skyFrame coordSystem skyFormat
	  {fr->markerRulerSystemCmd($1, (CoordSystem)$4, (SkyFrame)$5,
	    (CoordSystem)$6, (SkyFormat)$7);}

	| INT SELECT_ {fr->markerSelectCmd($1);}
	| INT SELECT_ ONLY_ {fr->markerSelectOnlyCmd($1);}
	| INT TAG_ STRING {fr->markerTagCmd($1,$3);}
	| INT TEXT_ STRING {fr->markerTextCmd($1,$3);}
	| INT TEXT_ ROTATE_ yesno {fr->markerTextRotateCmd($1,$4);}
	| INT UNHIGHLITE_ {fr->markerUnhighliteCmd($1);}
	| INT UNSELECT_ {fr->markerUnselectCmd($1);}

	| INT VECTOR_ ARROW_ yesno {fr->markerVectorArrowCmd($1,$4);}
	| INT VECTOR_ POINT_ internalSystem coord numeric angle
	  {fr->markerVectorCmd($1, Vector($5), (InternalSystem)$4, $6, $7);}
	| INT VECTOR_ POINT_ coordSystem skyFrame coord 
	    coordSystem skyFormat numeric angle
	  {fr->markerVectorCmd($1, Vector($6), (CoordSystem)$4, (SkyFrame)$5,
	    $9, (CoordSystem)$7, (SkyFormat)$8, $10);}

	| INT WIDTH_ INT {fr->markerLineWidthCmd($1,$3);}

	| KEY_ {fr->markerKeyCmd();}
	| KEY_ numeric numeric {fr->markerKeyCmd(Vector($2,$3));}

	| LIST_ markerList
	| LOAD_ markerLoad
	| MOVE_ markerMoveSelected
	| PRESERVE_ yesno {fr->markerPreserveCmd($2);}
	| PROPERTY_ markerProperty yesno {fr->markerPropertyCmd($2,$3);}
	| PROPERTY_ markerProperty yesno numeric numeric
	  {fr->markerPropertyCmd($2,$3,Vector($4,$5));}
	| ROTATE_ BEGIN_ numeric numeric 
	  {fr->markerRotateBeginCmd(Vector($3,$4));}
	| ROTATE_ MOTION_ numeric numeric INT
	  {fr->markerRotateMotionCmd(Vector($3,$4),$5);}
	| ROTATE_ END_ {fr->markerRotateEndCmd();}
	| SAVE_ STRING markerFormat coordSystem skyFrame skyFormat yesno
	  {fr->markerSaveCmd($2, (Base::MarkerFormat)$3,
	    (CoordSystem)$4, (SkyFrame)$5, (SkyFormat)$6, $7);}
	| SAVE_ TEMPLATE_ STRING {fr->markerSaveTemplateCmd($3);}
	| SELECT_ markerSelect

	| SHOW_ markerShow

	| STRING COLOR_ STRING {fr->markerColorCmd($1,$3);}
	| STRING COPY_ {fr->markerCopyCmd($1);}
	| STRING DELETE_ {fr->markerDeleteCmd($1);}
	| STRING CUT_ {fr->markerCutCmd($1);}
	| STRING FONT_ STRING {fr->markerFontCmd($1,$3);}
	| STRING HIGHLITE_ {fr->markerHighliteCmd($1);}
	| STRING HIGHLITE_ ONLY_ {fr->markerHighliteOnlyCmd($1);}
	| STRING MOVE_ numeric numeric {fr->markerMoveCmd($1,Vector($3,$4));}
	| STRING MOVE_ FRONT_ {fr->markerFrontCmd($1);}
	| STRING MOVE_ BACK_ {fr->markerBackCmd($1);}
	| STRING MOVE_ TO_ internalSystem coord 
	  {fr->markerMoveToCmd($1,Vector($5),(InternalSystem)$4);}
	| STRING MOVE_ TO_ coordSystem skyFrame coord 
	  {fr->markerMoveToCmd($1,Vector($6),(CoordSystem)$4,(SkyFrame)$5);}
	| STRING PROPERTY_ markerProperty yesno 
	  {fr->markerPropertyCmd($1,$3,$4);}
	| STRING SELECT_ {fr->markerSelectCmd($1);}
	| STRING SELECT_ ONLY_ {fr->markerSelectOnlyCmd($1);}
	| STRING UNHIGHLITE_ {fr->markerUnhighliteCmd($1);}
	| STRING UNSELECT_ {fr->markerUnselectCmd($1);}

	| TAG_ EDIT_ STRING STRING {fr->markerTagEditCmd($3,$4);}
	| TAG_ DELETE_ STRING {fr->markerTagDeleteCmd($3);}
	| TAG_ DELETE_ ALL_ {fr->markerTagDeleteAllCmd();}
	| TAG_ STRING {fr->markerTagCmd($2);}
	| TAG_ UPDATE_ STRING {fr->markerTagUpdateCmd($3);}

	| PASTE_ {fr->markerPasteCmd();}
	| PASTE_ coordSystem skyFrame 
	  {fr->markerPasteCmd((CoordSystem)$2,(SkyFrame)$3);}
	| UNDO_ {fr->markerUndoCmd();}
	| UNHIGHLITE_ ALL_ {fr->markerUnhighliteAllCmd();}
	| UNSELECT_ ALL_ {fr->markerUnselectAllCmd();}
	| WIDTH_ INT {fr->markerLineWidthCmd($2);}
	;

markerCallBack : SELECT_ {$$ = CallBack::SELECTCB;}
	| UNSELECT_ {$$ = CallBack::UNSELECTCB;}
	| HIGHLITE_ {$$ = CallBack::HIGHLITECB;}
	| UNHIGHLITE_ {$$ = CallBack::UNHIGHLITECB;}
	| BEGIN_ MOVE_  {$$ = CallBack::MOVEBEGINCB;}
	| MOVE_  {$$ = CallBack::MOVECB;}
	| END_ MOVE_  {$$ = CallBack::MOVEENDCB;}
	| BEGIN_ EDIT_ {$$ = CallBack::EDITBEGINCB;}
	| EDIT_ {$$ = CallBack::EDITCB;}
	| END_ EDIT_ {$$ = CallBack::EDITENDCB;}
	| BEGIN_ ROTATE_ {$$ = CallBack::ROTATEBEGINCB;}
	| ROTATE_ {$$ = CallBack::ROTATECB;}
	| END_ ROTATE_ {$$ = CallBack::ROTATEENDCB;}
	| DELETE_ {$$ = CallBack::DELETECB;}
	| TEXT_ {$$ = CallBack::TEXTCB;}
	| COLOR_ {$$ = CallBack::COLORCB;}
	| WIDTH_ {$$ = CallBack::LINEWIDTHCB;}
	| PROPERTY_ {$$ = CallBack::PROPERTYCB;}
	| FONT_ {$$ = CallBack::FONTCB;}
	| KEY_ {$$ = CallBack::KEYCB;}
	| UPDATE_ {$$ = CallBack::UPDATECB;}
	;

markerCentroid : /* empty */ {fr->markerCentroidCmd();}
	| INT {fr->markerCentroidCmd($1);}
	| AUTO_ yesno {fr->markerCentroidAutoCmd($2);}
	| OPTION_ INT numeric {fr->markerCentroidOptionCmd($2,$3);}
	;

markerCreate : 
	  CIRCLE_ numeric numeric 
	    numeric 
	    markerProperties
	  {fr->createCircleCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    $4,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| ANNULUS_ numeric numeric 
	    numeric numeric INT 
	    markerProperties
	  {fr->createAnnulusCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    $4,$5,$6,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| CPANDA_ numeric numeric 
	    angle angle INT
	    numeric numeric INT 
	    markerProperties
	  {fr->createCpandaCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    $4,$5,$6,
	    $7,$8,$9,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| ELLIPSE_ numeric numeric 
	    numeric numeric 
	    optangle 
	    markerProperties
	  {fr->createEllipseCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5),
	    $6,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| ELLIPSEANNULUS_ numeric numeric 
	    numeric numeric numeric INT 
	    optangle 
	    markerProperties
	  {fr->createEllipseAnnulusCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5), 
	    Vector($6,$6*$4/$5),$7,
	    $8,
	    currentColor,currentDash,currentWidth,currentFont, 
	    currentText,currentProps,NULL,taglist,cblist);}
	| EPANDA_ numeric numeric 
	    angle angle INT
	    numeric numeric numeric INT 
	    optangle 
	    markerProperties
	  {fr->createEpandaCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    $4,$5,$6,
	    Vector($7,$8),
	    Vector($9,$9*$7/$8),$10,
	    $11,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| BOX_ numeric numeric 
	    numeric numeric 
	    optangle 
	    markerProperties
	  {fr->createBoxCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5),
	    $6,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| BOXANNULUS_ numeric numeric 
	    numeric numeric numeric INT 
	    optangle 
	    markerProperties
	  {fr->createBoxAnnulusCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5),Vector($6,$6*$4/$5),$7,
	    $8,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| BPANDA_ numeric numeric 
	    angle angle INT
	    numeric numeric numeric INT 
	    optangle 
	    markerProperties
	  {fr->createBpandaCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    $4,$5,$6,
	    Vector($7,$8),
	    Vector($9,$9*$7/$8),$10,
	    $11,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| LINE_ numeric numeric 
	    numeric numeric 
	    markerProperties
	  {fr->createLineCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5)*fr->getCanvasToRef(),
	    0, 0,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| VECTOR_ numeric numeric 
	    numeric numeric 
	    markerProperties
	  {fr->createVectCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5)*fr->getCanvasToRef(),
	    1,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| PROJECTION_ numeric numeric 
	    numeric numeric 
	    numeric 
	    markerProperties
	  {fr->createProjectionCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5)*fr->getCanvasToRef(),
	    $6,
	    NULL,NULL,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| PROJECTION_ numeric numeric 
	    numeric numeric 
	    numeric 
	    STRING STRING 
	    markerProperties
	  {fr->createProjectionCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5)*fr->getCanvasToRef(),
	    $6,
	    $7,$8,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| RULER_ numeric numeric 
	    numeric numeric 
	    coordSystem skyFrame coordSystem skyFormat
	    markerProperties
	  {fr->createRulerCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5)*fr->getCanvasToRef(),
	    (CoordSystem)$6, (SkyFrame)$7, (CoordSystem)$8, (SkyFormat)$9,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| CIRCLE_ POINT_ numeric numeric pointSize
	    markerProperties
	  {fr->createCirclePointCmd(Vector($3,$4)*fr->getCanvasToRef(), $5,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| BOX_ POINT_ numeric numeric pointSize
	    markerProperties
	  {fr->createBoxPointCmd(Vector($3,$4)*fr->getCanvasToRef(), $5,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| DIAMOND_ POINT_ numeric numeric pointSize
	    markerProperties
	  {fr->createDiamondPointCmd(Vector($3,$4)*fr->getCanvasToRef(), $5,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| CROSS_ POINT_ numeric numeric pointSize
	    markerProperties
	  {fr->createCrossPointCmd(Vector($3,$4)*fr->getCanvasToRef(), $5,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| 'X' POINT_ numeric numeric pointSize
	    markerProperties
	  {fr->createExPointCmd(Vector($3,$4)*fr->getCanvasToRef(), $5,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| ARROW_ POINT_ numeric numeric pointSize
	    markerProperties
	  {fr->createArrowPointCmd(Vector($3,$4)*fr->getCanvasToRef(), $5,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}
	| BOXCIRCLE_ POINT_ numeric numeric pointSize
	    markerProperties
	  {fr->createBoxCirclePointCmd(Vector($3,$4)*fr->getCanvasToRef(), $5,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| TEXT_ numeric numeric 
	    optangle 
	    markerProperties
	  {fr->createTextCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    $4, 1,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| POLYGON_ numeric numeric 
	    numeric numeric 
	    markerProperties
	  {fr->createPolygonCmd(Vector($2,$3)*fr->getCanvasToRef(),
	    Vector($4,$5),
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| COMPASS_ numeric numeric 
	    numeric 
	    coordSystem skyFrame
	    markerProperties
	  {fr->createCompassCmd(Vector($2,$3)*fr->getCanvasToRef(), 
	    $4,
	    "N", "E", 1, 1,
	    (CoordSystem)$5, (SkyFrame)$6,
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| COMPOSITE_ markerProperties
	  {fr->createCompositeCmd(
	    currentColor,currentDash,currentWidth,currentFont,
	    currentText,currentProps,NULL,taglist,cblist);}

	| TEMPLATE_ markerCreateTemplate
	;

markerCreateTemplate : STRING numeric numeric
	  {fr->createTemplateCmd(Vector($2,$3)*fr->getCanvasToRef(), $1);}
	| VAR_ STRING numeric numeric
	  {fr->createTemplateVarCmd(Vector($3,$4)*fr->getCanvasToRef(), $2);}
	| STRING coordSystem skyFrame coord
	  {fr->createTemplateCmd(Vector($4),(CoordSystem)$2,(SkyFrame)$3, $1);}
	;

markerDash : INT INT {$$[0] = $1; $$[1] = $2;}
	;

markerEdit : BEGIN_ numeric numeric INT 
	  {fr->markerEditBeginCmd(Vector($2,$3),$4);}
	| MOTION_ numeric numeric INT 
	  {fr->markerEditMotionCmd(Vector($2,$3),$4);}
	| END_ {fr->markerEditEndCmd();}
	;

markerFormat : DS9_ {$$ = Base::DS9;}
	| XML_ {$$ = Base::XML;}
	| CIAO_ {$$ = Base::CIAO;}
	| SAOTNG_ {$$ = Base::SAOTNG;}
	| SAOIMAGE_ {$$ = Base::SAOIMAGE;}
	| PROS_ {$$ = Base::PROS;}
	| XY_ {$$ = Base::RAWXY;}
	;

markerGet : CENTROID_ markerGetCentroid
	| COLOR_ {fr->getMarkerColorCmd();}
	| FONT_ {fr->getMarkerFontCmd();}
	| HANDLE_ numeric numeric {fr->getMarkerHandleCmd(Vector($2,$3));}
	| ID_ numeric numeric {fr->getMarkerIdCmd(Vector($2,$3));}
	| ID_ ALL_ {fr->getMarkerIdAllCmd();}
	| INT ANGLE_ {fr->getMarkerAngleCmd($1);}
	| INT ANGLE_ internalSystem  {fr->getMarkerAngleCmd($1);}
	| INT ANGLE_ coordSystem skyFrame
	  {fr->getMarkerAngleCmd($1,(CoordSystem)$3, (SkyFrame)$4);}
	| INT ANNULUS_ RADIUS_ internalSystem
	  {fr->getMarkerAnnulusRadiusCmd($1, (InternalSystem)$4);}
	| INT ANNULUS_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerAnnulusRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT BOXANNULUS_ RADIUS_ internalSystem
	  {fr->getMarkerBoxAnnulusRadiusCmd($1,(InternalSystem)$4);}
	| INT BOXANNULUS_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerBoxAnnulusRadiusCmd($1,(CoordSystem)$4,(SkyFormat)$5);}
	| INT BOX_ RADIUS_ internalSystem
	  {fr->getMarkerBoxRadiusCmd($1,(InternalSystem)$4);}
	| INT BOX_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerBoxRadiusCmd($1,(CoordSystem)$4, (SkyFormat)$5);}
	| INT BPANDA_ ANGLE_ {fr->getMarkerBpandaAnglesCmd($1);}
	| INT BPANDA_ ANGLE_ internalSystem {fr->getMarkerBpandaAnglesCmd($1);}
	| INT BPANDA_ ANGLE_ coordSystem skyFrame
	  {fr->getMarkerBpandaAnglesCmd($1, (CoordSystem)$4, (SkyFrame)$5);}
	| INT BPANDA_ RADIUS_ internalSystem
	  {fr->getMarkerBpandaRadiusCmd($1, (InternalSystem)$4);}
	| INT BPANDA_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerBpandaRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT CENTER_ internalSystem
	  {fr->getMarkerCenterCmd($1, (InternalSystem)$3);}
	| INT CENTER_ coordSystem skyFrame skyFormat 
	  {fr->getMarkerCenterCmd($1, (CoordSystem)$3, (SkyFrame)$4, 
	    (SkyFormat)$5);}
	| INT CIRCLE_ RADIUS_ internalSystem
	  {fr->getMarkerCircleRadiusCmd($1, (InternalSystem)$4);}
	| INT CIRCLE_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerCircleRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT COLOR_ {fr->getMarkerColorCmd($1);}
	| INT COMPASS_ ARROW_ {fr->getMarkerCompassArrowCmd($1);}
	| INT COMPASS_ LABEL_ {fr->getMarkerCompassLabelCmd($1);}
	| INT COMPASS_ RADIUS_ internalSystem
	  {fr->getMarkerCompassRadiusCmd($1, (InternalSystem)$4);}
	| INT COMPASS_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerCompassRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT COMPASS_ SYSTEM_ {fr->getMarkerCompassSystemCmd($1);}
	| INT COMPOSITE_ GLOBAL_ {fr->getMarkerCompositeCmd($1);}
	| INT CPANDA_ ANGLE_ {fr->getMarkerCpandaAnglesCmd($1);}
	| INT CPANDA_ ANGLE_ internalSystem {fr->getMarkerCpandaAnglesCmd($1);}
	| INT CPANDA_ ANGLE_ coordSystem skyFrame
	  {fr->getMarkerCpandaAnglesCmd($1, (CoordSystem)$4, (SkyFrame)$5);}
	| INT CPANDA_ RADIUS_ internalSystem
	  {fr->getMarkerCpandaRadiusCmd($1, (InternalSystem)$4);}
	| INT CPANDA_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerCpandaRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT ELLIPSE_ RADIUS_ internalSystem
	  {fr->getMarkerEllipseRadiusCmd($1, (InternalSystem)$4);}
	| INT ELLIPSE_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerEllipseRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT ELLIPSEANNULUS_ RADIUS_ internalSystem
	  {fr->getMarkerEllipseAnnulusRadiusCmd($1, (InternalSystem)$4);}
	| INT ELLIPSEANNULUS_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerEllipseAnnulusRadiusCmd($1, (CoordSystem)$4, 
	    (SkyFormat)$5);}
	| INT EPANDA_ ANGLE_ {fr->getMarkerEpandaAnglesCmd($1);}
	| INT EPANDA_ ANGLE_ internalSystem {fr->getMarkerEpandaAnglesCmd($1);}
	| INT EPANDA_ ANGLE_ coordSystem skyFrame
	  {fr->getMarkerEpandaAnglesCmd($1, (CoordSystem)$4, (SkyFrame)$5);}
	| INT EPANDA_ RADIUS_ internalSystem
	  {fr->getMarkerEpandaRadiusCmd($1, (InternalSystem)$4);}
	| INT EPANDA_ RADIUS_ coordSystem skyFormat
	  {fr->getMarkerEpandaRadiusCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT FONT_ {fr->getMarkerFontCmd($1);}
	| INT HIGHLITE_ {fr->getMarkerHighlitedCmd($1);}
	| INT LINE_ ARROW_ {fr->getMarkerLineArrowCmd($1);}
	| INT LINE_ LENGTH_ coordSystem skyFormat
	  {fr->getMarkerLineLengthCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT LINE_ POINT_ internalSystem
	  {fr->getMarkerLineCmd($1, (InternalSystem)$4);}
	| INT LINE_ POINT_ coordSystem skyFrame skyFormat
	  {fr->getMarkerLineCmd($1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	| INT MAP_ LENGTH_ numeric coordSystem skyFormat
	  {fr->getMarkerMapLenFromRefCmd($1, $4,
	    (CoordSystem)$5,(SkyFormat)$6);}
	| INT POINT_ SIZE_ {fr->getMarkerPointSizeCmd($1);}
	| INT PROJECTION_ STRING STRING STRING STRING
	  {fr->getMarkerProjectionCmd($1,$3,$4,$5,$6);}
	| INT PROJECTION_ POINT_ internalSystem
	  {fr->getMarkerProjectionPointsCmd($1, (InternalSystem)$4);}
	| INT PROJECTION_ POINT_ coordSystem skyFrame skyFormat
	  {fr->getMarkerProjectionPointsCmd($1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	| INT PROJECTION_ LENGTH_ coordSystem skyFormat
	  {fr->getMarkerProjectionLengthCmd($1, (CoordSystem)$4, 
	    (SkyFormat)$5);}
	| INT PROJECTION_ METHOD_ {fr->getMarkerProjectionMethodCmd($1);} 
	| INT PROJECTION_ THICK_ internalSystem
	  {fr->getMarkerProjectionWidthCmd($1, (InternalSystem)$4);}
	| INT PROJECTION_ THICK_ coordSystem skyFormat
	  {fr->getMarkerProjectionWidthCmd($1,(CoordSystem)$4,(SkyFormat)$5);}
	| INT PROJECTION_ SYSTEM_ {fr->getMarkerProjectionSystemCmd($1);}
	| INT PROPERTY_ {fr->getMarkerPropertyCmd($1);}
	| INT PROPERTY_ markerProperty {fr->getMarkerPropertyCmd($1,$3);}
	| INT RULER_ LENGTH_ coordSystem skyFormat
	  {fr->getMarkerRulerLengthCmd($1,(CoordSystem)$4,(SkyFormat)$5);}
	| INT RULER_ POINT_ internalSystem
	  {fr->getMarkerRulerPointCmd($1, (InternalSystem)$4);}
	| INT RULER_ POINT_ coordSystem skyFrame skyFormat
	  {fr->getMarkerRulerPointCmd($1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	| INT RULER_ SYSTEM_ {fr->getMarkerRulerSystemCmd($1);}
	| INT SELECT_ {fr->getMarkerSelectedCmd($1);}
	| INT TAG_ {fr->getMarkerTagCmd($1);}
	| INT TAG_ INT {fr->getMarkerTagCmd($1,$3);}
	| INT TEXT_ {fr->getMarkerTextCmd($1);}
	| INT TEXT_ ROTATE_ {fr->getMarkerTextRotateCmd($1);}
	| INT TYPE_ {fr->getMarkerTypeCmd($1);}
	| INT VECTOR_ ARROW_ {fr->getMarkerVectorArrowCmd($1);}
	| INT VECTOR_ LENGTH_ coordSystem skyFormat
	  {fr->getMarkerVectorLengthCmd($1, (CoordSystem)$4, (SkyFormat)$5);}
	| INT VECTOR_ POINT_ internalSystem
	  {fr->getMarkerVectorCmd($1, (InternalSystem)$4);}
	| INT VECTOR_ POINT_ coordSystem skyFrame skyFormat
	  {fr->getMarkerVectorCmd($1, (CoordSystem)$4,
	    (SkyFrame)$5, (SkyFormat)$6);}
	| INT WIDTH_ {fr->getMarkerLineWidthCmd($1);}
	| HIGHLITE_ markerGetHighlite
	| HIGHLITE_ NUMBER_ {fr->getMarkerHighlitedNumberCmd();}
	| POLYGON_ SEGMENT_ numeric numeric
	  {fr->getMarkerPolygonSegmentCmd(Vector($3,$4));}
	| PRESERVE_ {fr->getMarkerPreserveCmd();}
	| PROPERTY_ markerProperty {fr->getMarkerPropertyCmd($2);}
	| SELECT_ markerGetSelect
	| SELECT_ NUMBER_ {fr->getMarkerSelectedNumberCmd();}

	| SHOW_ markerGetShow

	| STRING COLOR_ {fr->getMarkerColorCmd($1);}
	| STRING FONT_ {fr->getMarkerFontCmd($1);}
	| STRING ID_ {fr->getMarkerIdCmd($1);}
	| STRING PROPERTY_ markerProperty {fr->getMarkerPropertyCmd($1,$3);}
	| STRING TAG_ {fr->getMarkerTagCmd($1);}
	| STRING TAG_ NUMBER_ {fr->getMarkerTagNumberCmd($1);}

	| TAG_ ALL_ {fr->getMarkerTagsCmd();}
	| TAG_ DEFAULT_ NAME_ {fr->getMarkerTagDefaultNameCmd();}

	| WIDTH_ {fr->getMarkerLineWidthCmd();}
	;

markerGetCentroid : AUTO_ {fr->getMarkerCentroidAutoCmd();}
        | OPTION_ {fr->getMarkerCentroidOptionCmd();}
	;

markerGetHighlite : /* empty */ {fr->getMarkerHighlitedCmd();}
	| numeric numeric {fr->getMarkerHighlitedCmd(Vector($1,$2));}
	;

markerGetSelect : /* empty */ {fr->getMarkerSelectedCmd();}
	| numeric numeric {fr->getMarkerSelectedCmd(Vector($1,$2));}
	;

markerGetShow : /* empty */ {fr->getMarkerShowCmd();}
	| TEXT_ {fr->getMarkerShowTextCmd();}
	;

markerInitProp : {
	  strncpy(currentFont, "helvetica 10 normal", 32);
	  strncpy(currentColor, "green", 16);
	  currentDash[0] = 8;
	  currentDash[1] = 3;
	  currentWidth = 1;
	  strncpy(currentText, "", 80);
	  currentProps = defaultProps;
	  taglist.deleteAll();
	  cblist.deleteAll();
	}
	;

markerLayer : /* empty */ {fr->markerLayerCmd(Base::USER);}
	| POINTER_ {fr->markerLayerCmd(Base::USER);}
	| USER_ {fr->markerLayerCmd(Base::USER);}
	| CATALOG_  {fr->markerLayerCmd(Base::CATALOG);}
	;

markerList : markerFormat coordSystem skyFrame skyFormat 
	    yesno markerQuery markerTags
	  {fr->markerListCmd((Base::MarkerFormat)$1,
	    (CoordSystem)$2, (SkyFrame)$3, (SkyFormat)$4, $5,
	    0, propQMask, propQValue, taglist);}
	| SELECT_ markerFormat coordSystem skyFrame skyFormat 
	    yesno markerQuery markerTags
	  {fr->markerListCmd((Base::MarkerFormat)$2, 
	    (CoordSystem)$3, (SkyFrame)$4, (SkyFormat)$5, $6,
	    1, propQMask, propQValue, taglist);}
	;

markerLoad : markerFormat STRING
	  {fr->markerLoadCmd((Base::MarkerFormat)$1,$2);}
	| markerFormat STRING coordSystem skyFrame
	  {fr->markerLoadCmd((Base::MarkerFormat)$1,$2,
	    (CoordSystem)$3,(SkyFrame)$4);}
	| markerFormat INT
	  {fr->markerLoadCmd((Base::MarkerFormat)$1,$2);}
	| markerFormat INT coordSystem skyFrame
	  {fr->markerLoadCmd((Base::MarkerFormat)$1,$2,
	    (CoordSystem)$3,(SkyFrame)$4);}
	| FITS_ STRING STRING markerDash INT STRING 
	  {fr->markerLoadFitsCmd($2, $3, $4, $5, $6);}
	;

markerMoveSelected : numeric numeric {fr->markerMoveCmd(Vector($1,$2));}
	| FRONT_ {fr->markerFrontCmd();}
	| BACK_ {fr->markerBackCmd();}
	| BEGIN_ numeric numeric {fr->markerMoveBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric {fr->markerMoveMotionCmd(Vector($2,$3));}
	| END_ {fr->markerMoveEndCmd();}
	| TO_ internalSystem coord 
	  {fr->markerMoveToCmd(Vector($3), (InternalSystem)$2);}
	| TO_ coordSystem skyFrame coord 
	  {fr->markerMoveToCmd(Vector($4), (CoordSystem)$2, (SkyFrame)$3);}
	;

markerProps : markerProps markerProp
	| markerProp
	;

markerProp : markerProperty '=' yesno {setProps(&currentProps,$1,$3);}
	| COLOR_ '=' STRING {strncpy(currentColor,$3,16);}
	| DASHLIST_ '=' INT INT {currentDash[0]=$3;currentDash[1]=$4;}
	| WIDTH_ '=' INT {currentWidth = $3;}
	| FONT_ '=' STRING {strncpy(currentFont,$3,32);}
	| TEXT_ '=' STRING {strncpy(currentText,$3,80);}
	| tag
	| callback
	;

markerProperty : NONE_ {$$ = Marker::NONE;}
	| SELECT_ {$$ = Marker::SELECT;}
	| HIGHLITE_ {$$ = Marker::HIGHLITE;}
	| DASH_ {$$ = Marker::DASH;}
	| FIXED_ {$$ = Marker::FIXED;}
	| EDIT_ {$$ = Marker::EDIT;}
	| MOVE_  {$$ = Marker::MOVE;}
	| ROTATE_ {$$ = Marker::ROTATE;}
	| DELETE_ {$$ = Marker::DELETE;}
	| INCLUDE_ {$$ = Marker::INCLUDE;}
	| SOURCE_ {$$ = Marker::SOURCE;}
	;

markerProperties : /* empty */ markerInitProp
	| markerInitProp markerProps
	;

markerQuery: /* empty */ {propQMask=0;propQValue=0;}
	| {propQMask=0;propQValue=0;} queries
	;

markerSelect : ALL_ {fr->markerSelectAllCmd();}
	| ONLY_ numeric numeric {fr->markerSelectOnlyCmd(Vector($2,$3));}
	| TOGGLE_ {fr->markerSelectToggleCmd();}
	| TOGGLE_ numeric numeric {fr->markerSelectToggleCmd(Vector($2,$3));}
	;

markerShow : yesno {fr->markerShowCmd($1);}
	| TEXT_ yesno {fr->markerShowTextCmd($2);}
	;

queries : queries query
	| query
	;

query	: markerProperty '=' yesno 
	  {propQMask |= $1; setProps(&propQValue,$1,$3);}
	;

markerTags: /* empty */ {taglist.deleteAll()}
	| {taglist.deleteAll()} tags
	;

tags	: tags tag
	| tag
	;

tag	: TAG_ '=' STRING {taglist.append(new Tag($3));}
	;

callback : CALLBACK_ '=' markerCallBack STRING STRING {cblist.append(
	new CallBack(fr->getInterp(),(CallBack::Type)$3,$4,$5));}
	;

mask	: CLEAR_ {fr->maskClearCmd();}
	| COLOR_ STRING {fr->maskColorCmd($2);}
	| MARK_ INT {fr->maskMarkCmd($2);}
	| TRANSPARENCY_ numeric {fr->maskTransparencyCmd($2);}
	;

orient	: 'X' {fr->orientCmd(XX);}
	| 'Y' {fr->orientCmd(YY);}
	| XY_ {fr->orientCmd(XY);}
	| NONE_ {fr->orientCmd(NORMAL);}
	;

pan	: numeric numeric numeric numeric 
	  {fr->panCmd(Vector($1,$2),Vector($3,$4));}
	| internalSystem numeric numeric
	    {fr->panCmd(Vector($2,$3), (InternalSystem)$1);}
	| coordSystem skyFrame coord
	    {fr->panCmd(Vector($3), (CoordSystem)$1, (SkyFrame)$2);}
	| TO_ panTo
	| BBOX_ numeric numeric {fr->panBBoxCmd(Vector($2,$3));}
	| MOTION_ panMotion
	| PRESERVE_ yesno {fr->panPreserveCmd($2);}
	;

panTo	: internalSystem numeric numeric
	    {fr->panToCmd(Vector($2,$3), (InternalSystem)$1);}
	| coordSystem skyFrame coord
	    {fr->panToCmd(Vector($3), (CoordSystem)$1, (SkyFrame)$2);}
	;

panMotion : BEGIN_ numeric numeric {fr->panBeginCmd(Vector($2,$3));}
	| numeric numeric {fr->panMotionCmd(Vector($1,$2));}
	| END_ numeric numeric {fr->panEndCmd(Vector($2,$3));}
	;

panner	: yesno {fr->pannerCmd($1);}
	| WCS_ wcsSystem skyFrame 
	  {fr->pannerCmd((CoordSystem)$2,(SkyFrame)$3);}
	| STRING INT INT {fr->pannerCmd($1, $2, $3);}
	| UPDATE_ {fr->updatePannerCmd();}
	;

postscript : COLORSPACE_ pscolorspace 
	  {fr->psColorSpaceCmd((Widget::PSColorSpace)$2);}
	| LEVEL_ INT {fr->psLevelCmd($2);}
	| RESOLUTION_ INT {fr->psResolutionCmd($2);}
	;

pscolorspace : BW_ {$$ = Widget::BW;}
	| GRAY_ {$$ = Widget::GRAY;}
	| RGB_ {$$ = Widget::RGB;}
	| CMYK_ {$$ = Widget::CMYK;}
	;

region	: HIGHLITE_ regionHighlite
	| SELECT_ regionSelect
	;

regionHighlite : BEGIN_ numeric numeric 
	  {fr->regionHighliteBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric 
	  {fr->regionHighliteMotionCmd(Vector($2,$3));}
	| END_ {fr->regionHighliteEndCmd();}
	| SHIFT_ END_ {fr->regionHighliteShiftEndCmd();}
	;

regionSelect : BEGIN_ numeric numeric 
	  {fr->regionSelectBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric {fr->regionSelectMotionCmd(Vector($2,$3));}
	| END_ {fr->regionSelectEndCmd();}
	| SHIFT_ END_ {fr->regionSelectShiftEndCmd();}
	;

renderMethod: MIP_ {$$ = Base::MIP;}
	| AIP_ {$$ = Base::AIP;}
	| THRESHOLD_ {$$ = Base::THRESHOLD;}
	| BLEND_ {$$ = Base::BLEND;}
	;

rgb	: CHANNEL_ STRING {fr->setRGBChannelCmd($2);}
	| SYSTEM_ coordSystem {fr->setRGBSystemCmd((CoordSystem)$2);}
	| VIEW_ INT INT INT {fr->setRGBViewCmd($2,$3,$4);}
	;

rotate	: numeric {fr->rotateCmd(degToRad($1));}
	| numeric DEGREES_ {fr->rotateCmd(degToRad($1));}
	| MOTION_ rotateMotion
	| TO_ numeric {fr->rotateToCmd(degToRad($2));}
	| TO_ numeric DEGREES_ {fr->rotateToCmd(degToRad($2));}
	;

rotateMotion : BEGIN_ {fr->rotateBeginCmd();}
	| numeric {fr->rotateMotionCmd(degToRad($1));}
	| END_ {fr->rotateEndCmd();}
	;

save	: ARRAY_ saveArray
	| FITS_ saveFits
	;

saveArray : FILE_ STRING endian 
	  {fr->saveArrayFileCmd($2, (FitsFile::ArchType)$3);}
	| CHANNEL_ STRING endian 
	  {fr->saveArrayChannelCmd($2, (FitsFile::ArchType)$3);}
	| SOCKET_ INT endian 
	  {fr->saveArraySocketCmd($2, (FitsFile::ArchType)$3);}
	;

saveFits: IMAGE_ saveFitsImage
	| TABLE_ saveFitsTable
	| RESAMPLE_ saveFitsResample
	;

saveFitsImage : FILE_ STRING compressType {fr->saveFitsImageFileCmd($2, $3);}
	| CHANNEL_ STRING compressType {fr->saveFitsImageChannelCmd($2, $3);}
	| SOCKET_ INT compressType {fr->saveFitsImageSocketCmd($2, $3);}
	;

saveFitsTable : FILE_ STRING compressType {fr->saveFitsTableFileCmd($2, $3);}
	| CHANNEL_ STRING compressType {fr->saveFitsTableChannelCmd($2, $3);}
	| SOCKET_ INT compressType {fr->saveFitsTableSocketCmd($2, $3);}
	;

saveFitsResample : FILE_ STRING compressType 
	  {fr->saveFitsResampleFileCmd($2, $3);}
	| CHANNEL_ STRING compressType 
	  {fr->saveFitsResampleChannelCmd($2, $3);}
	| SOCKET_ INT compressType 
	  {fr->saveFitsResampleSocketCmd($2, $3);}
	;

smooth	: smoothFunction INT {fr->smoothCmd((FitsImage::SmoothFunction)$1,$2);}
	| DELETE_ {fr->smoothDeleteCmd();}
	;

smoothFunction : BOXCAR_ {$$ = FitsImage::BOXCAR;}
	| GAUSSIAN_ {$$ = FitsImage::GAUSSIAN;}
	| TOPHAT_ {$$ = FitsImage::TOPHAT;}
	;

update	: /* empty */ {fr->updateFitsCmd(0);}
	| INT numeric numeric numeric numeric
	  {fr->updateFitsCmd($1,BBox($2,$3,$4,$5),0);}
	| NOW_ {fr->updateFitsCmd(1);}
	| NOW_ INT numeric numeric numeric numeric
	  {fr->updateFitsCmd($2,BBox($3,$4,$5,$6),1);}
	| FITS_ SLICE_ INT {fr->sliceCmd($3);}
	;

warp	: numeric numeric {fr->warpCmd(Vector($1,$2));}
	| TO_ numeric numeric {fr->warpToCmd(Vector($2,$3));}
	;

wcs	: ALIGN_ wcsAlign
	| RESET_ {fr->wcsResetCmd();}
	| REPLACE_ INT {fr->wcsReplaceCmd($2);}
	| REPLACE_ STRING {fr->wcsReplaceCmd($2);}
	| APPEND_ INT {fr->wcsAppendCmd($2);}
	| APPEND_ STRING {fr->wcsAppendCmd($2);}
	;

wcsAlign : INT wcsSystem skyFrame 
	  {fr->wcsAlignCmd($1, (CoordSystem)$2, (SkyFrame)$3);}
	| INT wcsSystem skyFrame POINTER
	  {fr->wcsAlignCmd($1, (CoordSystem)$2, (SkyFrame)$3, (FitsImage*)$4);}
	;

win32	: PRINT_ {
#ifdef _WIN32
	  fr->win32PrintCmd();
#endif
	}
	;

zoom	: numeric numeric {fr->zoomCmd(Vector($1,$2));}
	| numeric numeric ABOUT_ internalSystem numeric numeric
	  {fr->zoomAboutCmd(Vector($1,$2),Vector($5,$6),(InternalSystem)$4);}
	| numeric numeric ABOUT_ coordSystem skyFrame coord
	  {fr->zoomAboutCmd(Vector($1,$2),Vector($6),
	    (CoordSystem)$4,(SkyFrame)$5);}
	| TO_ zoomTo
	;

zoomTo	: FIT_	{fr->zoomToFitCmd();}
	| FIT_	numeric {fr->zoomToFitCmd($2);}
	| numeric numeric {fr->zoomToCmd(Vector($1,$2));}
	| numeric numeric ABOUT_ internalSystem numeric numeric
	  {fr->zoomToAboutCmd(Vector($1,$2),Vector($5,$6),
	    (InternalSystem)$4);}
	| numeric numeric ABOUT_ coordSystem skyFrame coord
	  {fr->zoomToAboutCmd(Vector($1,$2), Vector($6),
	    (CoordSystem)$4, (SkyFrame)$5);}
	;

%%

static void setProps(unsigned short* props, unsigned short prop, int value)
{
  if (value)
    *props |= prop;
  else
    *props &= ~prop;
}
