// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "pannerpseudo.h"

// Tk Canvas Widget Function Declarations

int PannerPseudoColorCreateProc(Tcl_Interp*, Tk_Canvas, Tk_Item*, int, 
				Tcl_Obj *const []);

// PannerPseudoColor Specs

static Tk_CustomOption tagsOption = {
  Tk_CanvasTagsParseProc, Tk_CanvasTagsPrintProc, NULL
};

static Tk_ConfigSpec pannerPseudoColorSpecs[] = {

  {TK_CONFIG_STRING, (char*)"-command", NULL, NULL, "panner",
   Tk_Offset(WidgetOptions, cmdName), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, (char*)"-x", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, x), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, (char*)"-y", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, y), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, (char*)"-width", NULL, NULL, "256",
   Tk_Offset(WidgetOptions, width), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, (char*)"-height", NULL, NULL, "256",
   Tk_Offset(WidgetOptions, height), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_ANCHOR, (char*)"-anchor", NULL, NULL, "nw",
   Tk_Offset(WidgetOptions, anchor), 0, NULL},
  {TK_CONFIG_CUSTOM, (char*)"-tags", NULL, NULL, NULL,
   0, TK_CONFIG_NULL_OK, &tagsOption},

  {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0, NULL},
};

// Tk Static Structure

static Tk_ItemType pannerPseudoColorType = {
  (char*)"pannerpseudocolor",         // name
  sizeof(WidgetOptions),        // size
  PannerPseudoColorCreateProc, // configProc
  pannerPseudoColorSpecs,      // configSpecs
  WidgetConfigProc,             // configProc
  WidgetCoordProc,              // coordProc
  WidgetDeleteProc,             // deleteProc
  WidgetDisplayProc,            // displayProc
  0,                            // alwaysRedraw
  WidgetPointProc,              // pointProc
  WidgetAreaProc,               // areaProc
  WidgetPostscriptProc,         // postscriptProc
  WidgetScaleProc,              // scaleProc
  WidgetTranslateProc,          // translateProc
  (Tk_ItemIndexProc*)NULL,      // indexProc
  WidgetICursorProc,            // icursorProc
  (Tk_ItemSelectionProc*)NULL,  // selectionProc
  (Tk_ItemInsertProc*)NULL,     // insertProc
  (Tk_ItemDCharsProc*)NULL,     // dCharsProc
  (Tk_ItemType*)NULL            // nextPtr
};

// Non-Member Functions

int PannerPseudoColor_Init(Tcl_Interp* interp)
{
  Tk_CreateItemType(&pannerPseudoColorType);
  return TCL_OK;
}

int PannerPseudoColorCreateProc(Tcl_Interp* interp, Tk_Canvas canvas, 
				Tk_Item* item, int argc, Tcl_Obj *const argv[])
{
  PannerPseudoColor* panner = new PannerPseudoColor(interp, canvas, item);

  // and set default configuration

  if (panner->configure(argc, (const char**)argv, 0) != TCL_OK) {
    delete panner;
    Tcl_AppendResult(interp, 
		     " error occured while creating pannerPseudoColor.", NULL);
    return TCL_ERROR;
  }

  return TCL_OK;
}

PannerPseudoColor::PannerPseudoColor(Tcl_Interp* i, Tk_Canvas c, 
				     Tk_Item* item) : Panner(i, c, item)
{
  configSpecs = pannerPseudoColorSpecs;  // panner configure options
}

void PannerPseudoColor::clearPixmap()
{
  XFillRectangle(display, pixmap, gc, 0, 0, options->width, options->height);
}
