// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "attribute.h"
#include "widget.h"

static char* black = (char*)"black";
static char* white = (char*)"white";
static char* red = (char*)"red";
static char* green = (char*)"green";
static char* blue = (char*)"blue";
static char* cyan = (char*)"cyan";
static char* magenta = (char*)"magenta";
static char* yellow = (char*)"yellow";

Attribute::Attribute(Widget* p) : parent(p)
{
  style_ = SOLID;
  width_ = 1;

  font_ = HELVETICA;
  size_ = 10;
  tkfont_ = Tk_GetFont(parent->getInterp(), parent->getTkwin(), "helvetica 10 normal");

  colour_ = WHITE;
  colorName_ = white;
  setColour(colour_);
}

Attribute::~Attribute()
{
  if (tkfont_)
    Tk_FreeFont(tkfont_);
}

void Attribute::setStyle(double v)
{
  switch ((int)v) {
  case SOLID:
  case DASH:
    break;
  default:
    return;
  }

  style_ = (Style)((int)v);
}

void Attribute::setWidth(double v)
{
  if (v>0)
    width_ = v;
  else
    width_ = 1;
}

void Attribute::setSize(double v)
{
  if (v >= 1) {
    size_ = (int)v;
    initFont();
  }
}

void Attribute::setFont(double v)
{
  switch ((Font)((int)v)) {
  case DEFAULT:
  case TIMES:
  case HELVETICA:
  case SYMBOL:
  case COURIER:

  case DEFAULTBOLD:
  case TIMESBOLD:
  case HELVETICABOLD:
  case SYMBOLBOLD:
  case COURIERBOLD:

  case DEFAULTITALIC:
  case TIMESITALIC:
  case HELVETICAITALIC:
  case SYMBOLITALIC:
  case COURIERITALIC:
    font_ = (Font)((int)(v));
    initFont();
    return;

  default:
    return;
  }
}

void Attribute::initFont()
{
  if (tkfont_)
    Tk_FreeFont(tkfont_);
  tkfont_ = NULL;

  ostringstream str;
  switch (font_) {
  case DEFAULT:
    str << "system " << size_ << " normal" << ends;
    break;
  case TIMES:
    str << "times " << size_ << " normal" << ends;
    break;
  case HELVETICA:
    str << "helvetica " << size_ << " normal" << ends;
    break;
  case SYMBOL:
    str << "symbol " << size_ << " normal" << ends;
    break;
  case COURIER:
    str << "courier " << size_ << " normal" << ends;
    break;

  case DEFAULTBOLD:
    str << "system " << size_ << " bold" << ends;
    break;
  case TIMESBOLD:
    str << "times " << size_ << " bold" << ends;
    break;
  case HELVETICABOLD:
    str << "helvetica " << size_ << " bold" << ends;
    break;
  case SYMBOLBOLD:
    str << "symbol " << size_ << " bold" << ends;
    break;
  case COURIERBOLD:
    str << "courier " << size_ << " bold" << ends;
    break;

  case DEFAULTITALIC:
    str << "system " << size_ << " italic" << ends;
    break;
  case TIMESITALIC:
    str << "times " << size_ << " italic" << ends;
    break;
  case HELVETICAITALIC:
    str << "helvetica " << size_ << " italic" << ends;
    break;
  case SYMBOLITALIC:
    str << "symbol " << size_ << " italic" << ends;
    break;
  case COURIERITALIC:
    str << "courier " << size_ << " italic" << ends;
    break;
  default:
    return;
  }

  tkfont_ = Tk_GetFont(parent->getInterp(), parent->getTkwin(), str.str().c_str());
}

void Attribute::setColour(double v)
{
  // check for color change
  if (colour_ == (Colour)((int)(v)))
    return;

  switch ((int)v) {
  case BLACK:
    colorName_ = black;
    break;
  case WHITE:
    colorName_ = white;
    break;
  case RED:
    colorName_ = red;
    break;
  case GREEN:
    colorName_ = green;
    break;
  case BLUE:
    colorName_ = blue;
    break;
  case CYAN:
    colorName_ = cyan;
    break;
  case MAGENTA:
    colorName_ = magenta;
    break;
  case YELLOW:
    colorName_ = yellow;
    break;
  default:
    return;
  }

  color_ = parent->getColor(colorName_);
  colour_ = (Colour)((int)(v));
}
