// Copyright (C) 1999-2009
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "truecolor8.h"
#include "util.h"

#ifdef _MACOSX
extern void macosxGetMasks(unsigned long*, unsigned long*, 
			   unsigned long*, unsigned long*);
#endif

TrueColor8::TrueColor8(Visual* visual)
{

#if !(_WIN32 || _MACOSX)
  rm_ = decodeMask((unsigned char)visual->red_mask, &rs_);
  gm_ = decodeMask((unsigned char)visual->green_mask, &gs_);
  bm_ = decodeMask((unsigned char)visual->blue_mask, &bs_);
#endif
#if _MACOSX
  // tk visual masks are bogus
  unsigned long red_mask, green_mask, blue_mask, alpha_mask;
  macosxGetMasks(&red_mask, &green_mask, &blue_mask, &alpha_mask);

  rm_ = decodeMask((unsigned char)red_mask, &rs_);
  gm_ = decodeMask((unsigned char)green_mask, &gs_);
  bm_ = decodeMask((unsigned char)blue_mask, &bs_);
#endif
#if _WIN32
  // windows masks are plain wrong
  rm_ = decodeMask((unsigned char)0x00FF0000, &rs_);
  gm_ = decodeMask((unsigned char)0x0000FF00, &gs_);
  bm_ = decodeMask((unsigned char)0x000000FF, &bs_);
#endif
}

unsigned char TrueColor8::decodeMask(unsigned char mask, int* s)
{
  *s=0;
  for (int i=0; i<8; i++, (*s)++) {
    if (mask & 0x80)
      break;
    mask <<= 1;
  }
  return mask;
}

void TrueColor8::encodeTrueColor(XColor* src, char* dest, XImage* ximage)
{
  *dest = 
    ((((unsigned char)src->blue)  & bm_) >> bs_) | 
    ((((unsigned char)src->green) & gm_) >> gs_) | 
    ((((unsigned char)src->red)   & rm_) >> rs_);
}

void TrueColor8::encodeTrueColor(unsigned char* src, XImage* ximage)
{
  int& width = ximage->width;
  int& height = ximage->height;
  char* data = XImageData(ximage);

  const unsigned char* ptr = src;
  for (int j=0; j<height; j++) {
    char* dest = data + j*ximage->bytes_per_line;

    for (int i=0; i<width; i++, dest++, ptr+=3)
      *dest =
	((ptr[0] & rm_) >> rs_) |
	((ptr[1] & gm_) >> gs_) |
	((ptr[2] & bm_) >> bs_);
  }
}

